#include<stdio.h>
#include <vector>
#include <fstream>
#include <sstream>
#include<X11/X.h>
#include<X11/Xlib.h>
#include<GL/glew.h>
#include<glm/glm.hpp>
#include<glm/gtc/matrix_transform.hpp>
#include<GL/gl.h>
#include<GL/glx.h>
#include<GL/glu.h>

using namespace glm;

typedef GLXContext (*glXCreateContextAttribsARBProc)(Display*, GLXFBConfig, GLXContext, Bool, const int*);
#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600

//グローバル変数
Display *g_dpy;
Window g_root;
XVisualInfo* g_vi;
Window g_win;
GLXContext g_ctx;
GLuint g_simpleShader=0;
GLuint g_vertexBuffer=0;
GLuint g_vao = 0;
GLuint g_texture = 0;

//関数プロトタイプ
bool InitOGL();
GLuint LoadShaders(const char * vertex_file_path, const char * fragment_file_path);
GLuint CreateTextureFromBMPFile(const char * imagepath);
void Render();

//
//
//メイン関数 X11ウィンドウ作成とメッセージループ
int main(int argc, char *argv[])
{
    //X11ウィンドウの作成
    {
        //ディスプレイの取得
        g_dpy = XOpenDisplay(NULL);
        if(!g_dpy)
        {
            printf("Xサーバーに接続できませんでしたn");
            return 0;
        }
        //ビジュアルの取得
        GLint att[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
        g_vi = glXChooseVisual(g_dpy, 0, att);
        if(g_vi == NULL)
        {
            printf("\マッチするビジュアルが見つかりませんでした\n");
            return 0;
        }
        //カラーマップの作成
        Colormap cmap;
        g_root = DefaultRootWindow(g_dpy);
        cmap = XCreateColormap(g_dpy, g_root, g_vi->visual, AllocNone);

        //X11ウィンドウを作成
        XSetWindowAttributes swa;
        swa.colormap = cmap;
        swa.event_mask =  KeyPressMask;
        g_win = XCreateWindow(g_dpy, g_root, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, g_vi->depth, InputOutput, g_vi->visual, CWColormap | CWEventMask, &swa);

        //作成したX11ウィンドウをマップする
        XMapWindow(g_dpy, g_win);
        //ウィンドウ名を指定
        XStoreName(g_dpy, g_win, "Texture Mapping");
    }

    //OpenGLコンテキスト作成とVAO,VBO作成
    InitOGL();

	//メッセージループ
	bool loop=true;
    while(loop)
    {
        while(XPending(g_dpy))
        {
            XEvent xev;
            XNextEvent(g_dpy, &xev);
            if(xev.type == KeyPress) loop=false;
        }
        Render();
        glXSwapBuffers(g_dpy, g_win);
    }
    //終了処理
    glXMakeCurrent(g_dpy, None, NULL);
    glXDestroyContext(g_dpy, g_ctx);
    XDestroyWindow(g_dpy, g_win);
    XCloseDisplay(g_dpy);

    return 0;
}
//
//
//OpenGLコンテキスト作成とVAO,VBO作成等
bool InitOGL()
{
    //OpenGLバージョンを指定してコンテキストを作成
    int numFBC = 0;
    GLint visualAtt[] =
    {
        GLX_RENDER_TYPE, GLX_RGBA_BIT,
        GLX_DRAWABLE_TYPE, GLX_WINDOW_BIT,
        GLX_DOUBLEBUFFER, True,
        GLX_RED_SIZE, 1,
        GLX_GREEN_SIZE, 1,
        GLX_BLUE_SIZE, 1,
        GLX_DEPTH_SIZE, 1,
        GLX_STENCIL_SIZE, 1,
        None
    };

    GLXFBConfig *fbc = glXChooseFBConfig(g_dpy, DefaultScreen(g_dpy), visualAtt, &numFBC);

    if (!fbc)
    {
        printf("フレームバッファ設定を取得できませんでした\n");
        return false;
    }
    glXCreateContextAttribsARBProc glXCreateContextAttribsARB = (glXCreateContextAttribsARBProc) glXGetProcAddress((const GLubyte *) "glXCreateContextAttribsARB");
    if (!glXCreateContextAttribsARB)
    {
        fprintf(stderr, "glXCreateContextAttribsARB関数が見つかりません\n");
        XFree(fbc);
        return false;
    }
     static int contextAttribs[] =
    {
        GLX_CONTEXT_MAJOR_VERSION_ARB, 4,//希望のメジャーバージョン番号を指定
        GLX_CONTEXT_MINOR_VERSION_ARB, 0,//希望のマイナーバージョン番号を指定
        GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
        None
    };
    //コンテキスト作成
    g_ctx = glXCreateContextAttribsARB(g_dpy, *fbc, NULL, True, contextAttribs);
    XFree(fbc);
    if (!g_ctx)
    {
        printf("コンテキスト作成に失敗しました\n");
        return false;
    }
    //このコンテキストをカレントにする
    glXMakeCurrent(g_dpy, g_win, g_ctx);

    //Init GLEW
	if (glewInit() != GLEW_OK)
	{
		printf("glew 初期化　失敗\n");
		return false;
	}
    //シェーダー作成
    g_simpleShader = LoadShaders("SimpleVS.glsl", "SimpleFS.glsl");
    //VAO
    glGenVertexArrays(1, &g_vao);
	glBindVertexArray(g_vao);
    //VBO
	GLfloat vertices[] =
	{
		-1.0f, -1.0f, 0.0f,0.0f,0.0f,1.0f, 0.0f,1.0f,
		1.0f, -1.0f, 0.0f,0.0f,0.0f,1.0f, 1.0f,1.0f,
		0.0f,  1.0f, 0.0f,0.0f,0.0f,1.0f, 0.5f,0.0f,
	};
	glGenBuffers(1, &g_vertexBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);

	//テクスチャーをファイルから作成
	g_texture = CreateTextureFromBMPFile("Grid.bmp");
	//テクスチャーのサンプラーとテクスチャーユニットを紐付ける
	GLuint texture = glGetUniformLocation(g_simpleShader, "g_textureSampler");
	glUniform1i(texture, 0);
}
//
//
//シェーダー読み込みコンパイル
GLuint LoadShaders(const char* vertexFilename, const char* fragmentFilename)
{
	GLuint vertexShader = glCreateShader(GL_VERTEX_SHADER);
	GLuint fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
	//バーテックスシェーダのコードを読み込む
	std::string VertexShaderCode;
	std::ifstream VertexShaderStream(vertexFilename, std::ios::in);
	if (VertexShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << VertexShaderStream.rdbuf();
		VertexShaderCode = sstr.str();
		VertexShaderStream.close();
	}
	else
	{
		printf("ファイルが見つかりません\n");
		return 0;
	}
	//フラグメントシェーダーのコードを読み込む
	std::string FragmentShaderCode;
	std::ifstream FragmentShaderStream(fragmentFilename, std::ios::in);
	if (FragmentShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << FragmentShaderStream.rdbuf();
		FragmentShaderCode = sstr.str();
		FragmentShaderStream.close();
	}
	GLint Result = GL_FALSE;
	int infoLength;
	//バーテックスシェーダをコンパイル
	char const * VertexSourcePointer = VertexShaderCode.c_str();
	glShaderSource(vertexShader, 1, &VertexSourcePointer, NULL);
	glCompileShader(vertexShader);

	glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(vertexShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(vertexShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//フラグメントシェーダーをコンパイル
	char const * FragmentSourcePointer = FragmentShaderCode.c_str();
	glShaderSource(fragmentShader, 1, &FragmentSourcePointer, NULL);
	glCompileShader(fragmentShader);

	glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(fragmentShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(fragmentShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//バーテックスシェーダーとフラグメントシェーダーを「プログラムオブジェクト」に関連付ける
	GLuint program = glCreateProgram();
	glAttachShader(program, vertexShader);
	glAttachShader(program, fragmentShader);
	glLinkProgram(program);

	glGetProgramiv(program, GL_LINK_STATUS, &Result);
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> ProgramErrorMessage(infoLength + 1);
		glGetProgramInfoLog(program, infoLength, NULL, &ProgramErrorMessage[0]);
		printf(&ProgramErrorMessage[0]);
	}

	return program;
}
//
//
//
GLuint CreateTextureFromBMPFile(const char * imagepath)
{
	FILE * fp=fopen(imagepath, "rb");
	if (!fp)
	{
		printf("ファイルを開けません\n");
		return 0;
	}
	//BMPヘッダーは54バイト
	unsigned char header[54];
	if (fread(header, 1, 54, fp) != 54)
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	//ビットマップであれば"BM"ヘッダーがあるはず
	if (header[0] != 'B' || header[1] != 'M')
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	// 24bitカラーかどうか
	if (*(int*)&(header[30]) != 0 || *(int*)&(header[28]) != 24)
	{
		printf("24ビットカラーではありません");
		return 0;
	}
	//ピクセルデータを読み込む
	unsigned int width = *(int*)&(header[18]);
	unsigned int height = *(int*)&(header[22]);
	unsigned int size = *(int*)&(header[34]);

	unsigned char* data = new unsigned char[size];
	fread(data, 1, size, fp);
	//この時点でもはやファイルは不要
	fclose(fp);

	//空（から）のテクスチャーを作成
	GLuint id;
	glGenTextures(1, &id);
	//テクスチャーをバインドして、これからそのテクスチャーにデータを送り込むことを指示する
	glBindTexture(GL_TEXTURE_2D, id);
	// ピクセルデータをテクスチャーに送り込む
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_BGR, GL_UNSIGNED_BYTE, data);
	//この時点でメモリー上のデータは不要になる
	delete data;
	//テクスチャーの仕様を設定
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glGenerateMipmap(GL_TEXTURE_2D);

	return id;
}
//
//
//描画
void Render()
{
    //ビューポートをセット
    XWindowAttributes gwa;
    XGetWindowAttributes(g_dpy, g_win, &gwa);
    glViewport(0, 0, gwa.width, gwa.height);
    //クリア
    glClearColor(0.0f, 0.0f, 0.4f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //これ以降使うシェーダーのセット
    glUseProgram(g_simpleShader);
    glBindVertexArray(g_vao);
    //VBOをセット
    glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
    //シェーダーに頂点内の「位置成分」の場所とサイズを教える
	GLint posAttrib = glGetAttribLocation(g_simpleShader, "g_position");
	glEnableVertexAttribArray(posAttrib);
	glVertexAttribPointer(posAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)0);
	//シェーダーに頂点内の「法線成分」の場所とサイズを教える
	GLint normalAttrib = glGetAttribLocation(g_simpleShader, "g_normal");
	glEnableVertexAttribArray(normalAttrib);
	glVertexAttribPointer(normalAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)(sizeof(GLfloat)*3));
	//シェーダーに頂点内の「テクスチャー座標成分」の場所とサイズを教える
	GLint uvAttrib = glGetAttribLocation(g_simpleShader, "g_uv");
	glEnableVertexAttribArray(uvAttrib);
	glVertexAttribPointer(uvAttrib, 2, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)(sizeof(GLfloat) * 6));

    //行列を渡す
    //ワールド行列
    mat4 world = mat4(1.0);
	static float angle = 0;
	angle+=0.01;
	world=rotate(world,angle,vec3(0,1,0));

	//ビュー行列
	mat4 view = lookAt(vec3(0, 2, 2), vec3(0, 0, 0), vec3(0, 1, 0));
	//プロジェクション行列
	mat4 proj = perspectiveFovRH(45.0, (double)WINDOW_WIDTH, (double)WINDOW_HEIGHT, 0.1, 10000.0);
	//３つの合成行列をシェーダーに渡す
	GLint matrix = glGetUniformLocation(g_simpleShader, "g_worldViewProjMat");
	mat4 wvp=proj*view*world;
	glUniformMatrix4fv(matrix, 1, GL_FALSE, (GLfloat*)&wvp);
	//法線を回転させるためにワールド行列だけを個別に渡す
	GLint worldMatrix = glGetUniformLocation(g_simpleShader, "g_worldMat");
	glUniformMatrix4fv(worldMatrix, 1, GL_FALSE, (GLfloat*)&(world));
	//シェーダーにライトの方向を渡す
	GLint light = glGetUniformLocation(g_simpleShader, "g_lightVector");
	glUniform4f(light, 0,0,1,0);
	//テクスチャーをセットする
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, g_texture);
	//描画
    glDrawArrays(GL_TRIANGLES, 0, 3);
}
