#version 330 core
#define MAX_BONE 100

layout(location = 0) in vec4 g_position;
layout(location = 1) in vec4 g_normal;
layout(location = 2) in vec2 g_uv;
layout(location = 3) in uvec4 g_boneIndices;//ボーンの番号
layout(location = 4) in vec4 g_boneWeights;//ボーンの重み

uniform mat4 g_worldViewProjMat;
uniform mat4 g_boneMatrices[MAX_BONE];

out vec4 g_normalOut;
out vec2 g_uvQut;

struct Skin
{
	vec4 pos;
	vec4 normal;
	uvec4 boneIndices;
	vec4 boneWeights;
};

//指定した番号のボーンのポーズ行列を返す　サブ関数（バーテックスシェーダーで使用）
mat4 FetchBoneMatrix(uint idx)
{
	return g_boneMatrices[idx];
}
//頂点をスキニング（ボーンにより移動）するサブ関数
Skin SkinVert(Skin Input)
{
	Skin Output;
	Output.pos = vec4(0, 0, 0, 0);
	Output.normal = vec4(0, 0, 0, 0);

	//ボーン0
	mat4 m = FetchBoneMatrix(Input.boneIndices.x);
	Output.pos += Input.boneWeights.x * m*Input.pos;
	Output.normal += Input.boneWeights.x * m*Input.normal;
	//ボーン1
	m = FetchBoneMatrix(Input.boneIndices.y);
	Output.pos += Input.boneWeights.y * m*Input.pos;
	Output.normal += Input.boneWeights.y * m*Input.normal;
	//ボーン2
	m = FetchBoneMatrix(Input.boneIndices.z);
	Output.pos += Input.boneWeights.z * m*Input.pos;
	Output.normal += Input.boneWeights.z * m*Input.normal;
	//ボーン3
	m = FetchBoneMatrix(Input.boneIndices.w);
	Output.pos += Input.boneWeights.w * m*Input.pos;
	Output.normal += Input.boneWeights.w * m*Input.normal;

	return Output;
}
//
void main()
{
    Skin skin;
    skin.pos = g_position;
	skin.normal = g_normal;
	skin.boneIndices = g_boneIndices;
	skin.boneWeights = g_boneWeights;

	Skin skinned=SkinVert(skin);

	gl_Position = g_worldViewProjMat * skinned.pos;

	g_normalOut = g_normal;

	g_uvQut = g_uv;

}
