﻿#include <stdio.h>
#include <string.h>
#include<GL/glew.h>
#include "external/glm/glm.hpp"
#include "external/glm/gtc/matrix_transform.hpp"
#include "external/glm/gtx/matrix_interpolation.hpp"
#include "external/glm/gtc/quaternion.hpp"

#include <GL/gl.h>

using namespace glm;
/*
ここでのスキンメッシュは、ワンメッシュとする（ジオメトリのフレームが1つ、あとのフレームは全てボーンという構成を想定）
*/
struct X_VERTEX
{
	vec3 vPos;//頂点位置
	vec3 vNorm;//頂点法線
	vec2 vTex;//UV座標
	int bBoneIndex[4];//ボーン　番号
	float bBoneWeight[4];//ボーン　重み
	int iNumBones;//影響をうけるボーン数
	X_VERTEX()
	{
		memset(this,0,sizeof(X_VERTEX));
		for(int i=0;i<4;i++) bBoneIndex[i]=-1;
	}
};
struct X_MATERIAL
{
	char szName[100];
	vec4 Kd;//ディフューズ
	float Kp;//スペキュラーのパワー
	vec4 Ks;//スペキュラー
	vec4 Ke;//エミッシブ
	char szTextureName[100];//テクスチャーファイル名
	unsigned int dwNumFace;//そのマテリアルであるポリゴン数
	X_MATERIAL()
	{
		memset(this,0,sizeof(X_MATERIAL));
	}
};
//ボーン
struct X_BONE
{
	char name[256];
	int iMyIndex;//自分のインデックス
	mat4 mTransform;
	mat4 mBindPose;
	unsigned int dwNumChild;//子の数
	X_BONE* pChild[50];//自分の子へのポインター 50個まで

	//アニメーション情報
	unsigned int dwNumKey;//キーフレームの数　（入れ物の”フレーム”とは異なる）
	float* pfKeyTime;//キーフレームごとの時間
	mat4* pmKeyMatrix;//キーフレームごとの姿勢行列

	X_BONE()
	{
		memset(this,0,sizeof(X_BONE));
		mTransform=mat4(1.0);
		mBindPose=mat4(1.0);
	}
	~X_BONE()
	{
		delete[] pfKeyTime;
		delete[] pmKeyMatrix;
	}
};

mat4 g_mOrientation;//FrameTransformMatrix
unsigned int g_dwNumVert=0;
unsigned int g_dwNumFace=0;
unsigned int g_dwNumUV=0;
unsigned int g_dwNumMaterial=0;
X_MATERIAL* g_pMaterial=NULL;
int* g_piMateriaList=NULL;
unsigned int g_dwNumBones=0;
unsigned int g_dwBoneIndex=0;
X_BONE* g_pRootBone=NULL;//ルートボーン　全てのボーンの親。これを出発して、これのリンクを辿れば、全てのボーンを巡れる。
X_VERTEX* pvVertexBuffer=NULL;
int * g_piFace=NULL;

//
//
//
void RecursivePrintHierarchy(FILE* fp,X_BONE* pBone)
{
	fprintf(fp,"%s   //////////\n",pBone->name);
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		fprintf(fp,"   ");
		RecursivePrintHierarchy(fp,pBone->pChild[i]);
	}
	fprintf(fp,"%s   /////end\n",pBone->name);
}
//
//
//
void PrintHierarchy(FILE* fp)
{
	RecursivePrintHierarchy(fp,g_pRootBone);
}
//
//
//
bool LoadMesh(FILE* fp,int iReadStart,int iReadEnd)
{
	float x,y,z;
	int v1=0,v2=0,v3=0;
	int vn1=0,vn2=0,vn3=0;
	int vt1=0,vt2=0,vt3=0;
	int cnt=0;

	char key[400]={0};
	char str[400]={0};

	//マテリアル読み込み
	vec4 vV4;
	vec3 vV3;
	//マテリアル数を調べる
	fseek(fp,iReadStart,SEEK_SET);
	while(ftell(fp)<iReadEnd)
	{
		fscanf(fp,"%s ",key,sizeof(key));
		if(strcmp(key,"Material")==0)
		{
			g_dwNumMaterial++;
		}
	}
	//マテリアルのメモリー確保
	g_pMaterial=new X_MATERIAL[g_dwNumMaterial];

	fseek(fp,iReadStart,SEEK_SET);
	cnt=0;
	while(ftell(fp)<iReadEnd)
	{
		fscanf(fp,"%s ",key,sizeof(key));
		if(strcmp(key,"Material")==0)
		{
			fgets(str,sizeof(str),fp);//直後の行にあると推定　改行する
			//Kd　ディフューズ
			fscanf(fp,"%f;%f;%f;%f;;",&vV4.x,&vV4.y,&vV4.z,&vV4.w);
			g_pMaterial[cnt].Kd=vV4;
			//パワー（スペキュラーのハイライト）
			fscanf(fp,"%f;",&g_pMaterial[cnt].Kp);
			//Ks　スペキュラー
			fscanf(fp,"%f;%f;%f;;",&vV3.x,&vV3.y,&vV3.z);
			g_pMaterial[cnt].Ks=vec4(vV3,1);
			//Ke　エミッシブ
			fscanf(fp,"%f;%f;%f;;",&vV3.x,&vV3.y,&vV3.z);
			g_pMaterial[cnt].Ke=vec4(vV3,1);

			//テクスチャー
			fscanf(fp,"%s ",key,sizeof(key));
			if(strcmp(key,"TextureFilename")==0)
			{
				fgets(str,sizeof(str),fp);//直後の行にあると推定　改行する
				//map_Kd　テクスチャー
				fscanf(fp,"%s",g_pMaterial[cnt].szTextureName,sizeof(g_pMaterial[cnt].szTextureName));
				//2重引用符を除去
				int k=0;
				while( g_pMaterial[cnt].szTextureName[++k]!='\"') g_pMaterial[cnt].szTextureName[k-1]=g_pMaterial[cnt].szTextureName[k];
				g_pMaterial[cnt].szTextureName[k-1]=0;
			}
			cnt++;
		}
	}
	//事前に頂点数、ポリゴン数を調べる
	fseek(fp,iReadStart,SEEK_SET);
	while(ftell(fp)<iReadEnd)
	{
		//キーワード読み込み
		fscanf(fp,"%s ",key,sizeof(key));
		//頂点
		if(strcmp(key,"Mesh")==0)
		{
			fgets(str,sizeof(str),fp);//直後の行にあると推定　改行する
			fgets(str,sizeof(str),fp);
			g_dwNumVert=atoi(str);
			//頂点数だけ改行して、フェイス（頂点インデックス）のデータ部分まで進む
			for(unsigned int i=0;i<g_dwNumVert ;i++)
			{
				fgets(str,sizeof(str),fp);//改行
			}
			//フェイス（ポリゴン、頂点インデックス）
			fgets(str,sizeof(str),fp);
			g_dwNumFace=atoi(str);
		}
	}

	//メモリ確保
	pvVertexBuffer =new X_VERTEX[ g_dwNumVert ];
	g_piFace=new int[g_dwNumFace*3];//3頂点なので
	vec3* pvCoord=new vec3[ g_dwNumVert ];
	vec3* pvNormal=new vec3[ g_dwNumVert  ];
	vec2* pvTexture=new vec2[ g_dwNumVert ];

	//本読み込み
	fseek(fp,iReadStart,SEEK_SET);
	while(ftell(fp)<iReadEnd)
	{
		//キーワード 読み込み
		memset(key,0,sizeof(key));
		fscanf(fp,"%s ",key,sizeof(key));

		//頂点 読み込み
		if(strcmp(key,"Mesh")==0)
		{
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する

			for(unsigned int i=0;i<g_dwNumVert ;i++)
			{
				fscanf(fp,"%f;%f;%f;,",&x,&y,&z);
				pvCoord[i].x=x;
				pvCoord[i].y=y;
				pvCoord[i].z=z;
			}
			//フェイス読み込み
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する
			for(unsigned int i=0;i<g_dwNumFace ;i++)
			{
				int dammy;
				fscanf(fp,"%d;%d,%d,%d;,",&dammy,&v1,&v2,&v3);
				g_piFace[i*3]=v1;
				g_piFace[i*3+1]=v2;
				g_piFace[i*3+2]=v3;
			}
		}

		//法線 読み込み
		if(strcmp(key,"MeshNormals")==0)
		{
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する

			for(unsigned int i=0;i<g_dwNumVert ;i++)
			{
				fscanf(fp,"%f;%f;%f;,",&x,&y,&z);
				pvNormal[i].x=x;
				pvNormal[i].y=y;
				pvNormal[i].z=z;
			}
		}

		//テクスチャー座標 読み込み
		if(strcmp(key,"MeshTextureCoords")==0)
		{
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する
			fgets(str,sizeof(str),fp);//データは2行下にあると推定　改行する

			for(unsigned int i=0;i<g_dwNumVert ;i++)
			{
				fscanf(fp,"%f;%f;,",&x,&y);
				pvTexture[i].x=x;
				pvTexture[i].y=y;//Y成分が逆な場合があるので注意（CGソフト依存） 適宜1-yとすること
			}
		}
	}

	//マテリアルリスト
	fseek(fp,iReadStart,SEEK_SET);
	while(ftell(fp)<iReadEnd)
	{
		memset(key,0,sizeof(key));
		fscanf(fp,"%s ",key,sizeof(key));
		if(strcmp(key,"MeshMaterialList")==0)
		{
			fgets(str,sizeof(str),fp);//空改行する
			fgets(str,sizeof(str),fp);//空改行する

			fgets(str,sizeof(str),fp);//リスト要素数
			int iNumFace=atoi(str);
			g_piMateriaList=new int[iNumFace];
			for(int i=0;i<iNumFace;i++)
			{
				fgets(str,sizeof(str),fp);
				g_piMateriaList[i]=atoi(str);
			}
		}
	}

	//頂点構造体にコピー
	for(unsigned int i=0;i<g_dwNumVert ;i++)
	{
		pvVertexBuffer[i].vPos=pvCoord[i];
		pvVertexBuffer[i].vNorm=pvNormal[i];
		pvVertexBuffer[i].vTex=pvTexture[i];
	}
	//一時的な入れ物は、もはや不要
	delete[] pvCoord;
	delete[] pvNormal;
	delete[] pvTexture;

	return true;
}

//
//
//
int AddBoneHierarchy(FILE* fp,X_BONE* pParentBone,X_BONE* pBone,int iReadStart,int iReadEnd)
{
	pBone->iMyIndex=g_dwBoneIndex;
	g_dwBoneIndex++;

	char key[400];

	int iCurrent=0;
	//ボーン名（フレーム名）
	 fscanf(fp,"%s ",key,sizeof(key));
	 strcpy(pBone->name,key);

	while( strcmp(key,"FrameTransformMatrix")!=0 && !feof(fp)) fscanf(fp,"%s ",key,sizeof(key));
	fgets(key,sizeof(key),fp);//改行する
	mat4 m;
	m = mat4(1.0);

	fscanf(fp,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f;;",
		&m[0][0],&m[1][0],&m[2][0],&m[3][0],
		&m[0][1], &m[1][1], &m[2][1], &m[3][1],
		&m[0][2], &m[1][2], &m[2][2], &m[3][2],
		&m[0][3], &m[1][3], &m[2][3], &m[3][3]
		);
	pBone->mTransform=m;

	int stack=2;

	while(ftell(fp)<iReadEnd)
	{
		fscanf(fp,"%s ",key,sizeof(key));
		if(key[0]=='{')
		{
			stack++;
		}
		if(key[0]=='}')
		{
			stack--;
		}
		if(stack==0)//フレーム内　探索終了
		{
			while( strcmp(key,"Frame")!=0 && !feof(fp))
			{
				int hold=ftell(fp);
				fscanf(fp,"%s ",key,sizeof(key));
				//if(strcmp(key,"}")==0)//このボーンの兄弟ボーンはない。
				if(key[0]=='}')
				{
					fseek(fp,hold,0);
					return 0;
				}
			}
			iCurrent=ftell(fp);
			if(iReadEnd<=ftell(fp))
			{
				//このボーンの兄弟ボーンはない。
				break;
			}
			else
			{
				//兄弟ボーン
				pParentBone->dwNumChild++;
				pParentBone->pChild[pParentBone->dwNumChild]=new X_BONE;
				fseek(fp,iCurrent,0);
				AddBoneHierarchy(fp,pParentBone,pParentBone->pChild[pParentBone->dwNumChild],iCurrent,iReadEnd);
				break;
			}
			return 0;
		}
		if(stack<0)
		{
			return -1;//中括弧対応エラー
		}
		//子フレーム発見（子ボーン）
		if(strcmp(key,"Frame")==0)
		{
			pBone->pChild[pBone->dwNumChild]=new X_BONE;
			AddBoneHierarchy(fp,pBone,pBone->pChild[pBone->dwNumChild],iReadStart,iReadEnd);
			pBone->dwNumChild++;
		}
	}
	return 0;
}
//
//
//
X_BONE* GetBoneByName(char* szName,X_BONE* pBone)
{
	X_BONE* ret=NULL;
	if(strcmp(pBone->name,szName)==0)
	{
		return pBone;
	}
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		ret=GetBoneByName(szName,pBone->pChild[i]);
		if(ret != NULL) return ret;
	}
	return ret;
}
//
//
//
bool LoadBones(FILE* fp,int iReadStart,int iReadEnd)
{
	char key[400];

	//ルートボーン作成（内容は無関係）
	g_pRootBone=new X_BONE;
	strcpy(g_pRootBone->name,"root_bone");
	g_pRootBone->iMyIndex=-1;
	//ボーン同士の親子関係を調べる
	fseek(fp,iReadStart,0);
	g_pRootBone->pChild[0]=new X_BONE;
	AddBoneHierarchy(fp,g_pRootBone,g_pRootBone->pChild[0],iReadStart,iReadEnd);
	g_pRootBone->dwNumChild++;
	//ボーン数を読む
	fseek(fp,0,0);
	while( strcmp(key,"XSkinMeshHeader")!=0 && !feof(fp))
	{
		if(strcmp(key,"template")==0 ) fgets(key,sizeof(key),fp);//　改行する
		fscanf(fp,"%s ",key,sizeof(key));
	}
	fgets(key,sizeof(key),fp);//　改行する
	fgets(key,sizeof(key),fp);//　改行する
	fgets(key,sizeof(key),fp);//　改行する
	fgets(key,sizeof(key),fp);
	g_dwNumBones=atoi(key);
	//ボーンインデックスとボーンの重みを読み込む
	fseek(fp,0,0);
	for(unsigned int iBone=0;iBone<g_dwNumBones;iBone++)
	{
		while( strcmp(key,"SkinWeights")!=0 && !feof(fp))
		{
			if(strcmp(key,"template")==0 ) fgets(key,sizeof(key),fp);//　改行する
			fscanf(fp,"%s ",key,sizeof(key));
		}
		fgets(key,sizeof(key),fp);//改行する
		//名前
		char name[256];
		fscanf(fp,"%s;",name,sizeof(name));
		//2重引用符を除去
		int k=0;
		while( name[++k]!='\"') name[k-1]=name[k];
		name[k-1]=0;
		//その名前のボーンを探す
		X_BONE* pBone=GetBoneByName(name,g_pRootBone);
		if(!pBone) continue;

		fgets(key,sizeof(key),fp);//改行する
		int iNumWeight=0;
		fscanf(fp,"%d;",&iNumWeight);
		int* piIndex=new int[iNumWeight];
		float* pfWeight=new float[iNumWeight];
		for(int i=0;i<iNumWeight;i++)
		{
			fscanf(fp,"%d,",&piIndex[i]);
		}
		fgets(key,sizeof(key),fp);//改行する
		for(int i=0;i<iNumWeight;i++)
		{
			fscanf(fp,"%f,",&pfWeight[i]);
		}
		for(int i=0;i<iNumWeight;i++)
		{
			if(piIndex[i]==25)
			{
				int a=8;
			}
			//4つの中で最小のウェイトと比較
			int minIndex=0;
			float min=10;
			for(int k=0;k<4;k++)
			{
				if(pvVertexBuffer[piIndex[i]].bBoneWeight[k]<min)
				{
					min=pvVertexBuffer[piIndex[i]].bBoneWeight[k];
					minIndex=k;
				}
			}
			if(pfWeight[i]>pvVertexBuffer[piIndex[i]].bBoneWeight[minIndex])
			{
				pvVertexBuffer[piIndex[i]].bBoneIndex[minIndex]=pBone->iMyIndex;
				pvVertexBuffer[piIndex[i]].bBoneWeight[minIndex]=pfWeight[i];
			}
		}
		//バインドポーズ行列を読みこむ
		fgets(key,sizeof(key),fp);//改行する
		mat4 m;
		m=mat4(1.0);
		fscanf(fp,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f;;",
			&m[0][0], &m[0][1], &m[0][2], &m[0][3],
			&m[1][0], &m[1][1], &m[1][2], &m[1][3],
			&m[2][0], &m[2][1], &m[2][2], &m[2][3],
			&m[3][0], &m[3][1], &m[3][2], &m[3][3]
		);
		pBone->mBindPose=m;

		delete[] piIndex;
		delete[] pfWeight;
	}

	//ボーン未対応頂点チェック　フレームをボーンにしている可能性あり　その頂点だけ欠ける。　応急処置としてボーン0に対応させる
	for(int i=0;i<g_dwNumVert;i++)
	{
		int val=pvVertexBuffer[i].bBoneIndex[0]+pvVertexBuffer[i].bBoneIndex[1]+pvVertexBuffer[i].bBoneIndex[2]+pvVertexBuffer[i].bBoneIndex[3];
		if(val==-4)
		{
			pvVertexBuffer[i].iNumBones=1;
			pvVertexBuffer[i].bBoneIndex[0]=0;
			pvVertexBuffer[i].bBoneWeight[0]=1;
		}
	}
	return true;
}

//
//
//
bool LoadAnimationFromX(char* szFileName)
{
	char key[400];
	FILE* fp=NULL;
	fp=fopen(szFileName,"rt");
	if(!fp)
	{
		printf("ファイルを開けません\n");
		return false;
	}


	while(!feof(fp))
	{
		//Animationタグまで行く。
		while( strcmp(key,"Animation")!=0 && !feof(fp))
		{
			fscanf(fp,"%s ",key,sizeof(key));
		}
		//アニメーション名　ボーン名と対応
		fgets(key,sizeof(key),fp);//　改行する
		fscanf(fp,"%s ",key,sizeof(key));
		fscanf(fp,"%s ",key,sizeof(key));

		X_BONE* pBone=GetBoneByName(key,g_pRootBone);
		if(pBone==NULL) continue;

		//行列を読む
		while( strcmp(key,"AnimationKey")!=0 && !feof(fp))
		{
			fscanf(fp,"%s ",key,sizeof(key));
		}
		fgets(key,sizeof(key),fp);//　改行する
		fgets(key,sizeof(key),fp);//　改行する
		fscanf(fp,"%d",&pBone->dwNumKey);//キーの個数
		pBone->pfKeyTime=new float[pBone->dwNumKey];//キー時間メモリ確保
		pBone->pmKeyMatrix=new mat4[pBone->dwNumKey];//キー行列メモリ確保
		fgets(key,sizeof(key),fp);//　改行する
		int dammy;
		for(unsigned int i=0;i<pBone->dwNumKey;i++)
		{
			fscanf(fp,"%f;%d;%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f;;,",
				&pBone->pfKeyTime[i],&dammy,

				&pBone->pmKeyMatrix[i][0][0], &pBone->pmKeyMatrix[i][0][1], &pBone->pmKeyMatrix[i][0][2], &pBone->pmKeyMatrix[i][0][3],
				&pBone->pmKeyMatrix[i][1][0], &pBone->pmKeyMatrix[i][1][1], &pBone->pmKeyMatrix[i][1][2], &pBone->pmKeyMatrix[i][1][3],
				&pBone->pmKeyMatrix[i][2][0], &pBone->pmKeyMatrix[i][2][1], &pBone->pmKeyMatrix[i][2][2], &pBone->pmKeyMatrix[i][2][3],
				&pBone->pmKeyMatrix[i][3][0], &pBone->pmKeyMatrix[i][3][1], &pBone->pmKeyMatrix[i][3][2], &pBone->pmKeyMatrix[i][3][3]
			);
		}
	}
	fclose(fp);

	return true;
}
//
//
//
bool InitX(char* szFileName)
{
	int iReadStart=0;
	int iReadEnd=0;
	FILE* fp=NULL;
	fp=fopen(szFileName,"rt");
	if(!fp) return false;

	//最初のFrameタグまで行く。それをルートメッシュとする。空だが。
	char key[400];
	while( strcmp(key,"Frame")!=0 && !feof(fp))
	{
		if(strcmp(key,"template")==0 ) fgets(key,sizeof(key),fp);//　改行する
		fscanf(fp,"%s ",key,sizeof(key));
	}
	fgets(key,sizeof(key),fp);//改行する

	//次のFrameタグがジオメトリ（メッシュ）と推定
	while( strcmp(key,"Frame")!=0 && !feof(fp))
	{
		fscanf(fp,"%s ",key,sizeof(key));
	}
	iReadStart=ftell(fp);

	fgets(key,sizeof(key),fp);//改行する
	fgets(key,sizeof(key),fp);//改行する
	fgets(key,sizeof(key),fp);//改行する
	mat4 m;
	fscanf(fp,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",
		&m[0][0], &m[0][1], &m[0][2], &m[0][3],
		&m[1][0], &m[1][1], &m[1][2], &m[1][3],
		&m[2][0], &m[2][1], &m[2][2], &m[2][3],
		&m[3][0], &m[3][1], &m[3][2], &m[3][3]
	);
	g_mOrientation=m;
	fgets(key,sizeof(key),fp);//改行する
	fgets(key,sizeof(key),fp);//改行する

	while( strcmp(key,"Frame")!=0 && !feof(fp))
	{
		fscanf(fp,"%s ",key,sizeof(key));
	}
	iReadEnd=ftell(fp);

	LoadMesh(fp,iReadStart,iReadEnd);
	//これ以降のFrameタグは全てボーンとみなす
	iReadStart=iReadEnd;
	while( strcmp(key,"AnimationSet")!=0 && !feof(fp))
	{
		fscanf(fp,"%s ",key,sizeof(key));
	}
	iReadEnd=ftell(fp);

	LoadBones(fp,iReadStart,iReadEnd);

	//アニメーション情報を読み込む
	LoadAnimationFromX(szFileName);

	return true;
}

//
//
//
int GetNumVertices()
{
	return g_dwNumVert;
}

//
//
//
int GetNumFaces()
{
	return g_dwNumFace;
}
//
//
//
int GetNumMaterials()
{
	return g_dwNumMaterial;
}
//
//
//
char* GetTexturePath(int index)
{
	return g_pMaterial[index].szTextureName;
}
//
//
//
int GetNumUVs()
{
	return g_dwNumUV;
}


//
//
//
int GetNumBones()
{
	return g_dwNumBones;
}
//
//
//指定されたボーンが影響を及ぼす頂点数を返す
int GetNumBoneVertices(int index)
{
	int iNumInfluences=0;
	for(unsigned int i=0;i<g_dwNumVert;i++)
	{
		for(int k=0;k<4;k++)
		{
			if(pvVertexBuffer[i].bBoneIndex[k]==index)
			{
				iNumInfluences++;
				break;
			}
		}
	}
	return iNumInfluences;
}
//
//
//指定されたボーンが影響を及ぼす頂点のインデックスを返す 第2引数は、影響を受ける頂点のインデックスグループ内のインデックス（インデックスが若い順）
int GetBoneVerticesIndices(int iBoneIndex,int iIndexInGroup)
{
	int iIndexIndex=0;
	for(unsigned int i=0;i<g_dwNumVert;i++)
	{
		for(int k=0;k<4;k++)
		{
			if(pvVertexBuffer[i].bBoneIndex[k]==iBoneIndex)
			{
				if(iIndexIndex==iIndexInGroup)
				{
					return i;
				}
				iIndexIndex++;
			}
		}
	}
	return -1;
}
//
//
//指定されたボーンが影響を及ぼす頂点のボーンウェイトを返す 第2引数は、影響を受ける頂点のインデックスグループ内のインデックス（インデックスが若い順）
double GetBoneVerticesWeights(int iBoneIndex,int iIndexInGroup)
{
	int iIndexIndex=0;
	for(unsigned int i=0;i<g_dwNumVert;i++)
	{
		for(int k=0;k<4;k++)
		{
			if(pvVertexBuffer[i].bBoneIndex[k]==iBoneIndex)
			{
				if(iIndexIndex==iIndexInGroup)
				{
					return pvVertexBuffer[i].bBoneWeight[k];
				}
				iIndexIndex++;
			}
		}
	}
	return -1;
}
//
//
//
vec3 GetVertex(int iIndex)
{
	return pvVertexBuffer[iIndex].vPos;
}
//
//
//
vec3 GetNormal(int iIndex)
{
	return pvVertexBuffer[iIndex].vNorm;
}
//
//
//
vec2 GetUV(int iIndex)
{
	return pvVertexBuffer[iIndex].vTex;
}
//
//
//
int GetIndex(int iIndex)
{
	return g_piFace[iIndex];
}
//
//
//
vec4 GetAmbient(int iIndex)
{
	return g_pMaterial[iIndex].Ke;
}
//
//
//
vec4 GetDiffuse(int iIndex)
{
	return g_pMaterial[iIndex].Kd;
}
//
//
//
vec4 GetSpecular(int iIndex)
{
	return g_pMaterial[iIndex].Ks;
}
//
//
//
float GetSpecularPower(int iIndex)
{
	return g_pMaterial[iIndex].Kp;
}
//
//
//
int GetFaceVertexIndex(int iFaceIndex,int iIndexInFace)
{
	return g_piFace[iFaceIndex*3+iIndexInFace];
}
//
//
//
int GeFaceMaterialIndex(int iFaceIndex)
{
	return g_piMateriaList[iFaceIndex];
}


//
//2つの行列の間の線形補間を計算し、結果の行列を返す
//
mat4 Lerp(mat4 a, mat4 b,float t)
{
	mat4 ret;
	ret = interpolate(a, b, t);

	return ret;
}
//
//
//
mat4 RecursiveNewPose(bool* pbo,X_BONE* pBone,int iBoneIndex,int iCurrentFrame)
{
	mat4 m;
	m=mat4(1.0);
	*pbo=false;

	if(pBone->iMyIndex!=-1)
	{
		for(unsigned int i=0;i<pBone->dwNumKey-1;i++)
		{
			if( iCurrentFrame >=pBone->pfKeyTime[i] &&  iCurrentFrame <=pBone->pfKeyTime[i+1])
			{
				//2つのキー行列の間の線形補間をとる
				float length=pBone->pfKeyTime[i+1] - pBone->pfKeyTime[i];
				float current=iCurrentFrame - pBone->pfKeyTime[i];
				float t=current / length;

				m=Lerp(pBone->pmKeyMatrix[i], pBone->pmKeyMatrix[i+1],t);
				pBone->mTransform=m;//mTransformはそのときのフレーム時刻の変換行列で、つねに変化する（対してバインドポーズ行列は普遍）

				if(pBone->iMyIndex==iBoneIndex)
				{
					*pbo=true;
					return m;
				}
			}
		}
	}

	//子
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		bool bo=false;
		m= pBone->mTransform*RecursiveNewPose(&bo,pBone->pChild[i],iBoneIndex,iCurrentFrame);
		if(bo)
		{
			*pbo=true;
			return m;
		}
	}
	return m;
}

//
//
//
mat4 GetNewPose(int iBoneIndex,int iCurrentFrame)
{
	bool bo=false;
	return RecursiveNewPose(&bo,g_pRootBone,iBoneIndex,iCurrentFrame);
}
//
//
//
mat4 RecursiveFind(bool* pbo,X_BONE* pBone,int iBoneIndex)
{
	mat4 m;
	m = mat4(1.0);
	*pbo=false;
	if(pBone->iMyIndex==iBoneIndex)
	{
		m=pBone->mBindPose;
		*pbo=true;
		return m;
	}
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		bool bo=false;
		m=RecursiveFind(&bo,pBone->pChild[i],iBoneIndex);
		if(bo)
		{
			*pbo=true;
			return m;
		}
	}
	return m;
}
//
//
//
mat4 GetBindPose(int iBoneIndex)
{
	bool bo=false;
	return RecursiveFind(&bo,g_pRootBone,iBoneIndex);
}
//
//
//
int RecursiveGetNumChildBone(X_BONE* pBone,int iBoneIndex)
{
	int num=0;
	if(pBone->iMyIndex==iBoneIndex)
	{
		return pBone->dwNumChild;
	}
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		num=RecursiveGetNumChildBone(pBone->pChild[i],iBoneIndex);
		if(num !=-1) return num;
	}
	return -1;
}
//
//
//指定したボーンの子の数を返す。
int GetNumChildBone(int iBoneIndex)
{
	return RecursiveGetNumChildBone(g_pRootBone,iBoneIndex);
}
//
//
//
int RecursiveGetChildBoneIndex(X_BONE* pBone,int iBoneIndex,int iIndex)
{
	int num=0;
	if(pBone->iMyIndex==iBoneIndex)
	{
		return pBone->pChild[iIndex]->iMyIndex;
	}
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		num=RecursiveGetChildBoneIndex(pBone->pChild[i],iBoneIndex,iIndex);
		if(num !=-1) return num;
	}
	return -1;
}
//
//
//指定したボーンの子のインデックスを返す。第2引数はそのボーンのn番目の子という意味
int GetChildBoneIndex(int iBoneIndex,int iIndex)
{
	return RecursiveGetChildBoneIndex(g_pRootBone,iBoneIndex,iIndex);
}
//
//
//
char* RecursiveGetBoneName(X_BONE* pBone,int iBoneIndex)
{
	char* name=NULL;

	if(pBone->iMyIndex==iBoneIndex)
	{
		return pBone->name;
	}
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		name=RecursiveGetBoneName(pBone->pChild[i],iBoneIndex);
		if(name !=NULL) return name;
	}
	return NULL;
}
//
//
//
char* GetBoneName(int iBoneIndex)
{
	return RecursiveGetBoneName(g_pRootBone,iBoneIndex);
}
//
//
//
int GetNumBonesInVertex(int iVertexIndex)
{
	return pvVertexBuffer[iVertexIndex].iNumBones;
}
//
//
//
void DestroyBones(X_BONE* pBone)
{
	for(unsigned int i=0;i<pBone->dwNumChild;i++)
	{
		DestroyBones(pBone->pChild[i]);
	}
	delete pBone;
}
//
//
//
void DestroyX()
{
	delete[] g_piMateriaList;
	delete[] g_pMaterial;
	delete[] pvVertexBuffer;
	delete[] g_piFace;
	DestroyBones(g_pRootBone);
}
