#include<stdio.h>
#include <vector>
#include <fstream>
#include <sstream>
#include<X11/X.h>
#include<X11/Xlib.h>
#include<glm/glm.hpp>
#include<glm/gtc/matrix_transform.hpp>
#include<GL/gl.h>
#include<GL/glx.h>
#include<GL/glu.h>

using namespace glm;

#include<GL/glext.h>//(OpenGL)Eextensionsの定義が詰まっているglext.hをインクルード　ステップ①

//(OpenGL)Eextensions関数のポインターの受け皿を用意　ステップ②
PFNGLCREATEPROGRAMPROC glCreateProgram = NULL;
PFNGLGENVERTEXARRAYSPROC glGenVertexArrays = NULL;
PFNGLBINDVERTEXARRAYPROC glBindVertexArray = NULL;
PFNGLCREATESHADERPROC glCreateShader = NULL;
PFNGLSHADERSOURCEPROC glShaderSource = NULL;
PFNGLCOMPILESHADERPROC glCompileShader = NULL;
PFNGLGETSHADERIVPROC glGetShaderiv = NULL;
PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog = NULL;
PFNGLATTACHSHADERPROC glAttachShader = NULL;
PFNGLLINKPROGRAMPROC glLinkProgram = NULL;
PFNGLGETPROGRAMIVPROC glGetProgramiv = NULL;
PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog = NULL;
PFNGLGENBUFFERSPROC glGenBuffers = NULL;
PFNGLBINDBUFFERPROC glBindBuffer = NULL;
PFNGLBUFFERDATAPROC glBufferData = NULL;
PFNGLUSEPROGRAMPROC glUseProgram = NULL;
PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray = NULL;
PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer = NULL;
PFNGLGETATTRIBLOCATIONPROC glGetAttribLocation = NULL;
PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation = NULL;
PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv = NULL;
PFNGLUNIFORM4FPROC glUniform4f = NULL;
PFNGLUNIFORM1IARBPROC glUniform1i = NULL;
PFNGLVERTEXATTRIBIPOINTERPROC glVertexAttribIPointer = NULL;

typedef GLXContext (*glXCreateContextAttribsARBProc)(Display*, GLXFBConfig, GLXContext, Bool, const int*);
#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600

//グローバル変数
Display *g_dpy;
Window g_root;
XVisualInfo* g_vi;
Window g_win;
GLXContext g_ctx;
GLuint g_simpleShader=0;
GLuint g_vertexBuffer=0;
GLuint g_vao = 0;

//関数プロトタイプ
bool InitOGL();
GLuint LoadShaders(const char * vertex_file_path, const char * fragment_file_path);
void Render();

//
//
//メイン関数 X11ウィンドウ作成とメッセージループ
int main(int argc, char *argv[])
{
    //X11ウィンドウの作成
    {
        //ディスプレイの取得
        g_dpy = XOpenDisplay(NULL);
        if(!g_dpy)
        {
            printf("Xサーバーに接続できませんでしたn");
            return 0;
        }
        //ビジュアルの取得
        GLint att[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
        g_vi = glXChooseVisual(g_dpy, 0, att);
        if(g_vi == NULL)
        {
            printf("マッチするビジュアルが見つかりませんでした\n");
            return 0;
        }
        //カラーマップの作成
        Colormap cmap;
        g_root = DefaultRootWindow(g_dpy);
        cmap = XCreateColormap(g_dpy, g_root, g_vi->visual, AllocNone);

        //X11ウィンドウを作成
        XSetWindowAttributes swa;
        swa.colormap = cmap;
        swa.event_mask =  KeyPressMask;
        g_win = XCreateWindow(g_dpy, g_root, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, g_vi->depth, InputOutput, g_vi->visual, CWColormap | CWEventMask, &swa);

        //作成したX11ウィンドウをマップする
        XMapWindow(g_dpy, g_win);
        //ウィンドウ名を指定
        XStoreName(g_dpy, g_win, "Hello Transform");
    }

    //OpenGLコンテキスト作成とVAO,VBO作成
    InitOGL();

	//メッセージループ
	bool loop=true;
    while(loop)
    {
        while(XPending(g_dpy))
        {
            XEvent xev;
            XNextEvent(g_dpy, &xev);
            if(xev.type == KeyPress) loop=false;
        }
        Render();
        glXSwapBuffers(g_dpy, g_win);
    }
    //終了処理
    glXMakeCurrent(g_dpy, None, NULL);
    glXDestroyContext(g_dpy, g_ctx);
    XDestroyWindow(g_dpy, g_win);
    XCloseDisplay(g_dpy);

    return 0;
}
//
//
//OpenGLコンテキスト作成とVAO,VBO作成等
bool InitOGL()
{
    //OpenGLバージョンを指定してコンテキストを作成
    int numFBC = 0;
    GLint visualAtt[] =
    {
        GLX_RENDER_TYPE, GLX_RGBA_BIT,
        GLX_DRAWABLE_TYPE, GLX_WINDOW_BIT,
        GLX_DOUBLEBUFFER, True,
        GLX_RED_SIZE, 1,
        GLX_GREEN_SIZE, 1,
        GLX_BLUE_SIZE, 1,
        GLX_DEPTH_SIZE, 1,
        GLX_STENCIL_SIZE, 1,
        None
    };

    GLXFBConfig *fbc = glXChooseFBConfig(g_dpy, DefaultScreen(g_dpy), visualAtt, &numFBC);

    if (!fbc)
    {
        printf("フレームバッファ設定を取得できませんでした\n");
        return false;
    }
    glXCreateContextAttribsARBProc glXCreateContextAttribsARB = (glXCreateContextAttribsARBProc) glXGetProcAddress((const GLubyte *) "glXCreateContextAttribsARB");
    if (!glXCreateContextAttribsARB)
    {
        fprintf(stderr, "glXCreateContextAttribsARB関数が見つかりません\n");
        XFree(fbc);
        return false;
    }
     static int contextAttribs[] =
    {
        GLX_CONTEXT_MAJOR_VERSION_ARB, 4,//希望のメジャーバージョン番号を指定
        GLX_CONTEXT_MINOR_VERSION_ARB, 0,//希望のマイナーバージョン番号を指定
        GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
        None
    };
    //コンテキスト作成
    g_ctx = glXCreateContextAttribsARB(g_dpy, *fbc, NULL, True, contextAttribs);
    XFree(fbc);
    if (!g_ctx)
    {
        printf("コンテキスト作成に失敗しました\n");
        return false;
    }
    //このコンテキストをカレントにする
    glXMakeCurrent(g_dpy, g_win, g_ctx);

    //Init (OpenGL) Eextensions関数の実際のアドレスをポインターに代入して完了　ステップ③
	glCreateProgram = (PFNGLCREATEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glCreateProgram");
	glGenVertexArrays = (PFNGLGENVERTEXARRAYSPROC)glXGetProcAddress((const GLubyte*)"glGenVertexArrays");
	glBindVertexArray = (PFNGLBINDVERTEXARRAYPROC)glXGetProcAddress((const GLubyte*)"glBindVertexArray");
	glCreateShader = (PFNGLCREATESHADERPROC)glXGetProcAddress((const GLubyte*)"glCreateShader");
	glShaderSource = (PFNGLSHADERSOURCEPROC)glXGetProcAddress((const GLubyte*)"glShaderSource");
	glCompileShader = (PFNGLCOMPILESHADERPROC)glXGetProcAddress((const GLubyte*)"glCompileShader");
	glGetShaderiv = (PFNGLGETSHADERIVPROC)glXGetProcAddress((const GLubyte*)"glGetShaderiv");
	glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetShaderInfoLog");
	glAttachShader = (PFNGLATTACHSHADERPROC)glXGetProcAddress((const GLubyte*)"glAttachShader");
	glLinkProgram = (PFNGLLINKPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glLinkProgram");
	glGetProgramiv = (PFNGLGETPROGRAMIVPROC)glXGetProcAddress((const GLubyte*)"glGetProgramiv");
	glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetProgramInfoLog");
	glGenBuffers = (PFNGLGENBUFFERSPROC)glXGetProcAddress((const GLubyte*)"glGenBuffers");
	glBindBuffer = (PFNGLBINDBUFFERPROC)glXGetProcAddress((const GLubyte*)"glBindBuffer");
	glBufferData = (PFNGLBUFFERDATAPROC)glXGetProcAddress((const GLubyte*)"glBufferData");
	glUseProgram = (PFNGLUSEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glUseProgram");
	glEnableVertexAttribArray = (PFNGLENABLEVERTEXATTRIBARRAYPROC)glXGetProcAddress((const GLubyte*)"glEnableVertexAttribArray");
	glVertexAttribPointer = (PFNGLVERTEXATTRIBPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribPointer");
	glGetAttribLocation = (PFNGLGETATTRIBLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetAttribLocation");
    glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetUniformLocation");
	glUniformMatrix4fv = (PFNGLUNIFORMMATRIX4FVPROC)glXGetProcAddress((const GLubyte*)"glUniformMatrix4fv");
	glUniform4f = (PFNGLUNIFORM4FPROC)glXGetProcAddress((const GLubyte*)"glUniform4f");
	glUniform1i = (PFNGLUNIFORM1IARBPROC)glXGetProcAddress((const GLubyte*)"glUniform1i");
	glVertexAttribIPointer = (PFNGLVERTEXATTRIBIPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribIPointer");

    //シェーダー作成
    g_simpleShader = LoadShaders("SimpleVS.glsl", "SimpleFS.glsl");
    //VAO
    glGenVertexArrays(1, &g_vao);
	glBindVertexArray(g_vao);
    //VBO
	GLfloat vertices[] =
	{
		-1.0f, -1.0f, 0.0f,
		1.0f, -1.0f, 0.0f,
		0.0f,  1.0f, 0.0f,
	};
	glGenBuffers(1, &g_vertexBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
}
//
//
//シェーダー読み込みコンパイル
GLuint LoadShaders(const char* vertexFilename, const char* fragmentFilename)
{
	GLuint vertexShader = glCreateShader(GL_VERTEX_SHADER);
	GLuint fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
	//バーテックスシェーダのコードを読み込む
	std::string VertexShaderCode;
	std::ifstream VertexShaderStream(vertexFilename, std::ios::in);
	if (VertexShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << VertexShaderStream.rdbuf();
		VertexShaderCode = sstr.str();
		VertexShaderStream.close();
	}
	else
	{
		printf("ファイルが見つかりません\n");
		return 0;
	}
	//フラグメントシェーダーのコードを読み込む
	std::string FragmentShaderCode;
	std::ifstream FragmentShaderStream(fragmentFilename, std::ios::in);
	if (FragmentShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << FragmentShaderStream.rdbuf();
		FragmentShaderCode = sstr.str();
		FragmentShaderStream.close();
	}
	GLint Result = GL_FALSE;
	int infoLength;
	//バーテックスシェーダをコンパイル
	char const * VertexSourcePointer = VertexShaderCode.c_str();
	glShaderSource(vertexShader, 1, &VertexSourcePointer, NULL);
	glCompileShader(vertexShader);

	glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(vertexShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(vertexShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//フラグメントシェーダーをコンパイル
	char const * FragmentSourcePointer = FragmentShaderCode.c_str();
	glShaderSource(fragmentShader, 1, &FragmentSourcePointer, NULL);
	glCompileShader(fragmentShader);

	glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(fragmentShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(fragmentShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//バーテックスシェーダーとフラグメントシェーダーを「プログラムオブジェクト」に関連付ける
	GLuint program = glCreateProgram();
	glAttachShader(program, vertexShader);
	glAttachShader(program, fragmentShader);
	glLinkProgram(program);

	glGetProgramiv(program, GL_LINK_STATUS, &Result);
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> ProgramErrorMessage(infoLength + 1);
		glGetProgramInfoLog(program, infoLength, NULL, &ProgramErrorMessage[0]);
		printf(&ProgramErrorMessage[0]);
	}

	return program;
}
//
//
//描画
void Render()
{
    //ビューポートをセット
    XWindowAttributes gwa;
    XGetWindowAttributes(g_dpy, g_win, &gwa);
    glViewport(0, 0, gwa.width, gwa.height);
    //クリア
    glClearColor(0.0f, 0.0f, 0.4f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //これ以降使うシェーダーのセット
    glUseProgram(g_simpleShader);
    glBindVertexArray(g_vao);
    //VBOをセット
    glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
    //シェーダーに頂点内の「位置成分」の場所とサイズを教える
    GLint posAttrib = glGetAttribLocation(g_simpleShader, "g_position");
    glEnableVertexAttribArray(posAttrib);
    glVertexAttribPointer(posAttrib,3,GL_FLOAT,GL_FALSE,0,(void*)0);

    //行列を渡す
    //ワールド行列
    mat4 world = mat4(1.0);
	static float angle = 0;
	angle+=0.1;
	world=rotate(world,angle,vec3(0,1,0));

	//ビュー行列
	mat4 view = lookAt(vec3(3, 3, 3), vec3(0, 0, 0), vec3(0, 1, 0));
	//プロジェクション行列
	mat4 proj = perspectiveFovRH(45.0, (double)WINDOW_WIDTH, (double)WINDOW_HEIGHT, 0.1, 10000.0);
	//３つの合成行列をシェーダーに渡す
	GLint matrix = glGetUniformLocation(g_simpleShader, "g_worldViewProjMat");
	mat4 wvp=proj*view*world;
	glUniformMatrix4fv(matrix, 1, GL_FALSE, (GLfloat*)&wvp);
    //描画
    glDrawArrays(GL_TRIANGLES, 0, 3);
}
