#include<stdio.h>
#include <vector>
#include <fstream>
#include <sstream>
#include<X11/X.h>
#include<X11/Xlib.h>
#include<glm/glm.hpp>
#include<glm/gtc/matrix_transform.hpp>
#include<GL/gl.h>
#include<GL/glx.h>
#include<GL/glu.h>

using namespace glm;

#include<GL/glext.h>//(OpenGL)Eextensionsの定義が詰まっているglext.hをインクルード　ステップ①

//(OpenGL)Eextensions関数のポインターの受け皿を用意　ステップ②
PFNGLCREATEPROGRAMPROC glCreateProgram = NULL;
PFNGLGENVERTEXARRAYSPROC glGenVertexArrays = NULL;
PFNGLBINDVERTEXARRAYPROC glBindVertexArray = NULL;
PFNGLCREATESHADERPROC glCreateShader = NULL;
PFNGLSHADERSOURCEPROC glShaderSource = NULL;
PFNGLCOMPILESHADERPROC glCompileShader = NULL;
PFNGLGETSHADERIVPROC glGetShaderiv = NULL;
PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog = NULL;
PFNGLATTACHSHADERPROC glAttachShader = NULL;
PFNGLLINKPROGRAMPROC glLinkProgram = NULL;
PFNGLGETPROGRAMIVPROC glGetProgramiv = NULL;
PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog = NULL;
PFNGLGENBUFFERSPROC glGenBuffers = NULL;
PFNGLBINDBUFFERPROC glBindBuffer = NULL;
PFNGLBUFFERDATAPROC glBufferData = NULL;
PFNGLUSEPROGRAMPROC glUseProgram = NULL;
PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray = NULL;
PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer = NULL;
PFNGLGETATTRIBLOCATIONPROC glGetAttribLocation = NULL;
PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation = NULL;
PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv = NULL;
PFNGLUNIFORM4FPROC glUniform4f = NULL;
PFNGLUNIFORM1IARBPROC glUniform1i = NULL;
PFNGLVERTEXATTRIBIPOINTERPROC glVertexAttribIPointer = NULL;
PFNGLGENERATEMIPMAPPROC glGenerateMipmap = NULL;

typedef GLXContext (*glXCreateContextAttribsARBProc)(Display*, GLXFBConfig, GLXContext, Bool, const int*);
#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600

//グローバル変数
Display *g_dpy;
Window g_root;
XVisualInfo* g_vi;
Window g_win;
GLXContext g_ctx;
GLuint g_simpleShader=0;
GLuint g_vertexBuffer=0;
GLuint g_indexBuffer = 0;
GLuint g_vao = 0;
GLuint g_texture = 0;

//関数プロトタイプ
bool InitOGL();
GLuint LoadShaders(const char * vertex_file_path, const char * fragment_file_path);
GLuint CreateTextureFromBMPFile(const char * imagepath);
void Render();

//
//
//メイン関数 X11ウィンドウ作成とメッセージループ
int main(int argc, char *argv[])
{
    //X11ウィンドウの作成
    {
        //ディスプレイの取得
        g_dpy = XOpenDisplay(NULL);
        if(!g_dpy)
        {
            printf("Xサーバーに接続できませんでしたn");
            return 0;
        }
        //ビジュアルの取得
        GLint att[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
        g_vi = glXChooseVisual(g_dpy, 0, att);
        if(g_vi == NULL)
        {
            printf("\マッチするビジュアルが見つかりませんでした\n");
            return 0;
        }
        //カラーマップの作成
        Colormap cmap;
        g_root = DefaultRootWindow(g_dpy);
        cmap = XCreateColormap(g_dpy, g_root, g_vi->visual, AllocNone);

        //X11ウィンドウを作成
        XSetWindowAttributes swa;
        swa.colormap = cmap;
        swa.event_mask =  KeyPressMask;
        g_win = XCreateWindow(g_dpy, g_root, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, g_vi->depth, InputOutput, g_vi->visual, CWColormap | CWEventMask, &swa);

        //作成したX11ウィンドウをマップする
        XMapWindow(g_dpy, g_win);
        //ウィンドウ名を指定
        XStoreName(g_dpy, g_win, "Hand Gun (define in a source code)");
    }

    //OpenGLコンテキスト作成とVAO,VBO作成
    InitOGL();

	//メッセージループ
	bool loop=true;
    while(loop)
    {
        while(XPending(g_dpy))
        {
            XEvent xev;
            XNextEvent(g_dpy, &xev);
            if(xev.type == KeyPress) loop=false;
        }
        Render();
        glXSwapBuffers(g_dpy, g_win);
    }
    //終了処理
    glXMakeCurrent(g_dpy, None, NULL);
    glXDestroyContext(g_dpy, g_ctx);
    XDestroyWindow(g_dpy, g_win);
    XCloseDisplay(g_dpy);

    return 0;
}
//
//
//OpenGLコンテキスト作成とVAO,VBO作成等
bool InitOGL()
{
    //OpenGLバージョンを指定してコンテキストを作成
    int numFBC = 0;
    GLint visualAtt[] =
    {
        GLX_RENDER_TYPE, GLX_RGBA_BIT,
        GLX_DRAWABLE_TYPE, GLX_WINDOW_BIT,
        GLX_DOUBLEBUFFER, True,
        GLX_RED_SIZE, 1,
        GLX_GREEN_SIZE, 1,
        GLX_BLUE_SIZE, 1,
        GLX_DEPTH_SIZE, 1,
        GLX_STENCIL_SIZE, 1,
        None
    };

    GLXFBConfig *fbc = glXChooseFBConfig(g_dpy, DefaultScreen(g_dpy), visualAtt, &numFBC);

    if (!fbc)
    {
        printf("フレームバッファ設定を取得できませんでした\n");
        return false;
    }
    glXCreateContextAttribsARBProc glXCreateContextAttribsARB = (glXCreateContextAttribsARBProc) glXGetProcAddress((const GLubyte *) "glXCreateContextAttribsARB");
    if (!glXCreateContextAttribsARB)
    {
        fprintf(stderr, "glXCreateContextAttribsARB関数が見つかりません\n");
        XFree(fbc);
        return false;
    }
     static int contextAttribs[] =
    {
        GLX_CONTEXT_MAJOR_VERSION_ARB, 4,//希望のメジャーバージョン番号を指定
        GLX_CONTEXT_MINOR_VERSION_ARB, 0,//希望のマイナーバージョン番号を指定
        GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
        None
    };
    //コンテキスト作成
    g_ctx = glXCreateContextAttribsARB(g_dpy, *fbc, NULL, True, contextAttribs);
    XFree(fbc);
    if (!g_ctx)
    {
        printf("コンテキスト作成に失敗しました\n");
        return false;
    }
    //このコンテキストをカレントにする
    glXMakeCurrent(g_dpy, g_win, g_ctx);

    //Init (OpenGL) Eextensions関数の実際のアドレスをポインターに代入して完了　ステップ③
	glCreateProgram = (PFNGLCREATEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glCreateProgram");
	glGenVertexArrays = (PFNGLGENVERTEXARRAYSPROC)glXGetProcAddress((const GLubyte*)"glGenVertexArrays");
	glBindVertexArray = (PFNGLBINDVERTEXARRAYPROC)glXGetProcAddress((const GLubyte*)"glBindVertexArray");
	glCreateShader = (PFNGLCREATESHADERPROC)glXGetProcAddress((const GLubyte*)"glCreateShader");
	glShaderSource = (PFNGLSHADERSOURCEPROC)glXGetProcAddress((const GLubyte*)"glShaderSource");
	glCompileShader = (PFNGLCOMPILESHADERPROC)glXGetProcAddress((const GLubyte*)"glCompileShader");
	glGetShaderiv = (PFNGLGETSHADERIVPROC)glXGetProcAddress((const GLubyte*)"glGetShaderiv");
	glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetShaderInfoLog");
	glAttachShader = (PFNGLATTACHSHADERPROC)glXGetProcAddress((const GLubyte*)"glAttachShader");
	glLinkProgram = (PFNGLLINKPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glLinkProgram");
	glGetProgramiv = (PFNGLGETPROGRAMIVPROC)glXGetProcAddress((const GLubyte*)"glGetProgramiv");
	glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetProgramInfoLog");
	glGenBuffers = (PFNGLGENBUFFERSPROC)glXGetProcAddress((const GLubyte*)"glGenBuffers");
	glBindBuffer = (PFNGLBINDBUFFERPROC)glXGetProcAddress((const GLubyte*)"glBindBuffer");
	glBufferData = (PFNGLBUFFERDATAPROC)glXGetProcAddress((const GLubyte*)"glBufferData");
	glUseProgram = (PFNGLUSEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glUseProgram");
	glEnableVertexAttribArray = (PFNGLENABLEVERTEXATTRIBARRAYPROC)glXGetProcAddress((const GLubyte*)"glEnableVertexAttribArray");
	glVertexAttribPointer = (PFNGLVERTEXATTRIBPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribPointer");
	glGetAttribLocation = (PFNGLGETATTRIBLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetAttribLocation");
    glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetUniformLocation");
	glUniformMatrix4fv = (PFNGLUNIFORMMATRIX4FVPROC)glXGetProcAddress((const GLubyte*)"glUniformMatrix4fv");
	glUniform4f = (PFNGLUNIFORM4FPROC)glXGetProcAddress((const GLubyte*)"glUniform4f");
	glUniform1i = (PFNGLUNIFORM1IARBPROC)glXGetProcAddress((const GLubyte*)"glUniform1i");
	glVertexAttribIPointer = (PFNGLVERTEXATTRIBIPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribIPointer");
   	glGenerateMipmap = (PFNGLGENERATEMIPMAPPROC)glXGetProcAddress((const GLubyte*)"glGenerateMipmap");
	glUniform1i = (PFNGLUNIFORM1IARBPROC)glXGetProcAddress((const GLubyte*)"glUniform1i");
	glVertexAttribIPointer = (PFNGLVERTEXATTRIBIPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribIPointer");

    //シェーダー作成
    g_simpleShader = LoadShaders("SimpleVS.glsl", "SimpleFS.glsl");
    //VAO
    glGenVertexArrays(1, &g_vao);
	glBindVertexArray(g_vao);
    //VBO
	GLfloat vertices[] =
	{
		//	position								normal								uv
		0.009890,0.033975,0.095500,	    1.000000,0.000000,0.000000,	    0.624919,0.846135,
		0.009890,0.033975,0.073000,	    1.000000,0.000000,0.000000,	    0.594146,0.846135,
		-0.009250,0.033975,0.073000,	    -1.000000,0.000000,0.000000,	    0.756628,0.846135,
		-0.009250,0.033975,0.095500,	    -1.000000,0.000000,0.000000,	    0.787401,0.846135,
		-0.009250,0.033975,0.020500,	    -1.000000,0.000000,0.000000,	    0.684824,0.846135,
		0.009890,0.033975,0.020500,	    1.000000,0.000000,0.000000,	    0.522343,0.846135,
		-0.009250,0.033975,0.013000,	    -1.000000,0.000000,0.000000,	    0.674567,0.846135,
		0.009890,0.033975,0.013000,	    1.000000,-0.000000,0.000000,	    0.512085,0.846135,
		-0.009250,0.033975,-0.018500,	    -1.000000,0.000000,0.000000,	    0.631484,0.846135,
		-0.009250,0.033975,-0.018500,	    0.000000,-0.902862,-0.429931,	    0.793871,0.845032,
		0.009890,0.033975,-0.018500,	    1.000000,-0.000000,0.000000,	    0.469003,0.846135,
		0.009890,0.033975,-0.018500,	    0.000000,-0.902862,-0.429931,	    0.816596,0.845032,
		0.009890,0.036975,0.032500,	    1.000000,-0.000000,0.000000,	    0.538755,0.845032,
		-0.009250,0.036975,0.032500,	    -1.000000,0.000000,0.000000,	    0.701237,0.845032,
		0.016660,-0.117150,0.091948,	    0.000000,-1.000000,0.000000,	    0.679057,0.821034,
		0.016660,-0.117150,0.091948,	    1.000000,0.000000,0.000000,	    0.696774,0.842948,
		0.016660,-0.117150,0.091948,	    0.000000,0.518828,-0.854878,	    0.696774,0.842948,
		0.016660,0.027850,0.075948,	    0.000000,-1.000000,0.000000,	    0.560642,0.842347,
		0.016660,0.027850,0.075948,	    0.000000,-0.189601,-0.981861,	    0.813053,0.775331,
		0.016660,0.027850,0.075948,	    0.707103,-0.134069,-0.694284,	    0.547432,0.032522,
		0.016660,0.027850,0.075948,	    0.707107,0.707107,0.000000,	    0.547432,0.032522,
		0.016660,0.027850,0.155948,	    0.000000,-1.000000,0.000000,	    0.451491,0.840253,
		0.016660,0.027850,0.155948,	    0.000000,-0.780868,0.624696,	    0.451491,0.840253,
		0.016660,0.027850,0.155948,	    0.000000,0.246767,0.969075,	    0.579069,0.768327,
		0.016660,0.027850,0.155948,	    0.707107,0.707107,0.000000,	    0.793760,0.032522,
		0.016660,0.027850,0.155948,	    0.707108,0.174491,0.685238,	    0.793760,0.032522,
		0.016660,0.027850,0.155948,	    0.707105,0.174491,0.685241,	    0.793760,0.032522,
		0.016660,-0.117150,0.187948,	    0.000000,-1.000000,0.000000,	    0.679057,0.785002,
		0.016660,-0.117150,0.187948,	    0.000000,-0.073370,0.997305,	    0.579069,0.821032,
		0.016660,-0.117150,0.187948,	    1.000000,0.000000,0.000000,	    0.779179,0.842948,
		-0.016660,-0.117150,0.091948,	    0.000000,-1.000000,0.000000,	    0.631444,0.821034,
		-0.016660,-0.117150,0.091948,	    0.000000,0.518828,-0.854878,	    0.781719,0.842948,
		-0.016660,-0.117150,0.091948,	    -1.000000,0.000000,0.000000,	    0.402380,0.821032,
		-0.016660,0.027850,0.075948,	    0.000000,-1.000000,0.000000,	    0.558858,0.870407,
		-0.016660,0.027850,0.075948,	    0.000000,-0.189601,-0.981861,	    0.781719,0.775331,
		-0.016660,0.027850,0.075948,	    -0.707107,0.707107,0.000000,	    0.436174,0.056180,
		-0.016660,0.027850,0.075948,	    -0.707103,-0.134069,-0.694284,	    0.436174,0.056180,
		-0.016660,0.027850,0.155948,	    0.000000,-1.000000,0.000000,	    0.449708,0.868312,
		-0.016660,0.027850,0.155948,	    0.000000,-0.780868,0.624696,	    0.449708,0.868312,
		-0.016660,0.027850,0.155948,	    0.000000,0.246767,0.969075,	    0.531455,0.768327,
		-0.016660,0.027850,0.155948,	    -0.707108,0.174491,0.685238,	    0.213761,0.056180,
		-0.016660,0.027850,0.155948,	    -0.707105,0.174491,0.685241,	    0.213761,0.056180,
		-0.016660,0.027850,0.155948,	    -0.707107,0.707107,0.000000,	    0.213761,0.056180,
		-0.016660,-0.117150,0.187948,	    0.000000,-1.000000,0.000000,	    0.631444,0.785002,
		-0.016660,-0.117150,0.187948,	    0.000000,-0.073370,0.997305,	    0.531455,0.821032,
		-0.016660,-0.117150,0.187948,	    -1.000000,0.000000,0.000000,	    0.527596,0.821032,
		0.013712,0.102850,0.075948,	    0.000000,1.000000,0.000000,	    0.623074,0.821034,
		0.013712,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    0.630986,0.771651,
		-0.013278,0.102850,0.075948,	    0.000000,1.000000,0.000000,	    0.587438,0.821034,
		-0.013278,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    0.592467,0.771651,
		-0.013278,0.102850,0.155948,	    0.000000,1.000000,0.000000,	    0.587438,0.791795,
		-0.013278,0.102850,0.155948,	    0.000000,0.990681,0.136201,	    0.587438,0.791795,
		0.013712,0.102850,0.155948,	    0.000000,1.000000,0.000000,	    0.623074,0.791795,
		0.013712,0.102850,0.155948,	    -0.000000,0.990681,0.136201,	    0.623074,0.791795,
		0.016660,0.027850,-0.177052,	    0.000000,-1.000000,0.000000,	    0.905831,0.848970,
		0.016660,0.027850,-0.177052,	    -0.000000,-0.554695,-0.832054,	    0.767540,0.821036,
		-0.016660,0.027850,-0.177052,	    0.000000,-1.000000,0.000000,	    0.904048,0.877030,
		-0.016660,0.027850,-0.177052,	    -0.000000,-0.554695,-0.832054,	    0.717710,0.821036,
		-0.022261,0.027850,0.155948,	    0.000000,-1.000000,0.000000,	    0.449408,0.873028,
		-0.022261,0.027850,0.155948,	    -0.994325,-0.106390,0.000000,	    0.878929,0.864528,
		-0.022261,0.027850,0.155948,	    0.988745,0.105793,0.105793,	    1.662891,0.753367,
		-0.022261,0.027850,0.155948,	    0.000000,-0.780868,0.624696,	    0.449408,0.873028,
		-0.017815,0.102850,0.155948,	    0.000000,1.000000,-0.000000,	    0.581449,0.791795,
		-0.017815,0.102850,0.155948,	    0.000000,0.990681,0.136201,	    0.581449,0.791795,
		-0.017815,0.102850,0.155948,	    -0.995315,0.095783,0.013168,	    0.878929,0.778784,
		-0.017815,0.102850,0.155948,	    -0.995401,0.095792,0.000000,	    0.878929,0.778784,
		-0.017815,0.102850,0.075948,	    0.000000,1.000000,0.000000,	    0.581449,0.821034,
		-0.017815,0.102850,0.075948,	    -0.995401,0.095792,0.000000,	    0.763319,0.778784,
		-0.017815,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    0.585993,0.771651,
		-0.017815,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    1.766603,0.460215,
		-0.022261,0.027850,0.075948,	    -0.994325,-0.106390,0.000000,	    0.763319,0.864528,
		-0.022261,0.027850,0.075948,	    0.000000,-1.000000,0.000000,	    0.558559,0.875123,
		0.022261,0.027850,0.155948,	    0.000000,-1.000000,0.000000,	    0.451791,0.835537,
		0.022261,0.027850,0.155948,	    0.993385,-0.114835,0.000000,	    0.878621,0.870781,
		0.022261,0.027850,0.155948,	    -0.986899,0.114085,0.114085,	    1.719229,0.753367,
		0.022261,0.027850,0.155948,	    0.000000,-0.780868,0.624696,	    0.451791,0.835537,
		0.022261,0.027850,0.075948,	    0.993385,-0.114835,0.000000,	    0.765395,0.870781,
		0.022261,0.027850,0.075948,	    0.000000,-1.000000,0.000000,	    0.560942,0.837631,
		0.018247,0.102850,0.075948,	    0.000000,1.000000,-0.000000,	    0.629063,0.821034,
		0.018247,0.102850,0.075948,	    0.996249,0.086538,0.000000,	    0.765394,0.786137,
		0.018247,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    0.637459,0.771651,
		0.018247,0.102850,0.075948,	    0.000000,0.000000,-1.000000,	    1.824599,0.460215,
		0.018247,0.102850,0.155948,	    0.000000,1.000000,-0.000000,	    0.629063,0.791795,
		0.018247,0.102850,0.155948,	    -0.000000,0.990681,0.136201,	    0.629063,0.791795,
		0.018247,0.102850,0.155948,	    0.996178,0.086532,0.011897,	    0.878621,0.786137,
		0.018247,0.102850,0.155948,	    0.996249,0.086538,-0.000000,	    0.878621,0.786137,
		-0.022261,0.027850,-0.177052,	    -0.994325,-0.106390,0.000000,	    0.397701,0.864528,
		-0.022261,0.027850,-0.177052,	    0.000000,-1.000000,0.000000,	    0.903748,0.881746,
		-0.022261,0.027850,-0.177052,	    0.000000,-0.554695,-0.832054,	    0.709335,0.821036,
		0.022261,0.027850,-0.177052,	    0.993385,-0.114835,0.000000,	    0.407315,0.870781,
		0.022261,0.027850,-0.177052,	    0.000000,-1.000000,0.000000,	    0.906131,0.844254,
		0.022261,0.027850,-0.177052,	    0.000000,-0.554695,-0.832054,	    0.775915,0.821036,
		-0.024186,0.045850,0.155948,	    -0.994325,-0.106390,0.000000,	    0.878929,0.843950,
		-0.024186,0.045850,0.155948,	    -1.000000,0.000000,0.000000,	    0.878929,0.843950,
		-0.024186,0.045850,0.155948,	    1.000000,-0.000000,0.000000,	    1.662891,0.727759,
		-0.024186,0.045850,0.155948,	    0.988745,0.105793,0.105793,	    1.662891,0.727759,
		0.024341,0.045850,0.155948,	    0.993385,-0.114835,0.000000,	    0.878621,0.850467,
		0.024341,0.045850,0.155948,	    1.000000,0.000000,0.000000,	    0.878621,0.850467,
		0.024341,0.045850,0.155948,	    -0.986899,0.114085,0.114085,	    1.719229,0.727759,
		0.024341,0.045850,0.155948,	    -1.000000,0.000000,0.000000,	    1.719229,0.727759,
		-0.024186,0.045850,0.075948,	    -1.000000,0.000000,0.000000,	    0.763319,0.843950,
		-0.024186,0.045850,0.075948,	    -0.994325,-0.106390,0.000000,	    0.763319,0.843950,
		0.024341,0.045850,0.075948,	    1.000000,0.000000,0.000000,	    0.765395,0.850467,
		0.024341,0.045850,0.075948,	    0.993385,-0.114835,0.000000,	    0.765395,0.850467,
		0.018237,0.045850,-0.189052,	    0.000000,-0.554695,-0.832054,	    0.769899,0.814645,
		0.018237,0.045850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.769899,0.814645,
		-0.018082,0.045850,-0.189052,	    -0.000000,-0.554695,-0.832054,	    0.715583,0.814645,
		-0.018082,0.045850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.715583,0.814645,
		-0.024186,0.045850,-0.189052,	    -1.000000,0.000000,0.000000,	    0.380359,0.843950,
		-0.024186,0.045850,-0.189052,	    -0.994325,-0.106390,0.000000,	    0.380359,0.843950,
		-0.024186,0.045850,-0.189052,	    0.000000,-0.554695,-0.832054,	    0.706455,0.814645,
		-0.024186,0.045850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.706455,0.814645,
		0.024341,0.045850,-0.189052,	    1.000000,0.000000,0.000000,	    0.390331,0.850467,
		0.024341,0.045850,-0.189052,	    0.993385,-0.114835,0.000000,	    0.390331,0.850467,
		0.024341,0.045850,-0.189052,	    0.000000,-0.554695,-0.832054,	    0.779027,0.814645,
		0.024341,0.045850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.779027,0.814645,
		-0.024186,0.059850,0.155948,	    -0.952475,0.304434,0.010573,	    0.878929,0.827944,
		-0.024186,0.059850,0.155948,	    -1.000000,0.000000,0.000000,	    0.878929,0.827944,
		-0.024186,0.059850,0.155948,	    -0.958012,0.286069,0.019450,	    0.878929,0.827944,
		-0.024186,0.059850,0.155948,	    -0.952528,0.304451,0.000000,	    0.878929,0.827944,
		0.024341,0.059850,0.155948,	    1.000000,0.000000,0.000000,	    0.878621,0.834666,
		0.024341,0.059850,0.155948,	    0.957864,0.286559,0.019483,	    0.878621,0.834666,
		0.024341,0.059850,0.155948,	    0.951622,0.307122,0.009536,	    0.878621,0.834666,
		0.024341,0.059850,0.155948,	    0.951666,0.307136,-0.000000,	    0.878621,0.834666,
		-0.024186,0.059850,0.075948,	    -0.952528,0.304451,0.000000,	    0.763319,0.827944,
		-0.024186,0.059850,0.075948,	    -1.000000,0.000000,0.000000,	    0.763319,0.827944,
		0.024341,0.059850,0.075948,	    0.951666,0.307136,-0.000000,	    0.765395,0.834666,
		0.024341,0.059850,0.075948,	    1.000000,0.000000,0.000000,	    0.765395,0.834666,
		-0.018082,0.059850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.715583,0.809675,
		0.018237,0.059850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.769899,0.809675,
		-0.024186,0.059850,-0.189052,	    -0.952528,0.304451,0.000000,	    0.380359,0.827944,
		-0.024186,0.059850,-0.189052,	    -1.000000,0.000000,0.000000,	    0.380359,0.827944,
		-0.024186,0.059850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.706455,0.809675,
		0.024341,0.059850,-0.189052,	    0.951666,0.307136,0.000000,	    0.390331,0.834666,
		0.024341,0.059850,-0.189052,	    1.000000,0.000000,0.000000,	    0.390331,0.834666,
		0.024341,0.059850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.779027,0.809675,
		0.018237,0.035850,0.165948,	    0.000000,-0.919176,0.393846,	    0.437932,0.838663,
		0.018237,0.035850,0.165948,	    0.000000,-0.780868,0.624696,	    0.437932,0.838663,
		0.018237,0.044964,0.187219,	    0.000000,-0.919176,0.393846,	    0.408910,0.838106,
		0.018237,0.044964,0.187219,	    -0.000000,0.426261,0.904600,	    0.844890,0.821039,
		0.018237,0.044964,0.187219,	    -0.000000,0.426261,0.904600,	    2.058350,0.668778,
		-0.018082,0.044964,0.187219,	    0.000000,-0.919176,0.393846,	    0.406966,0.868691,
		-0.018082,0.044964,0.187219,	    -0.000000,0.426261,0.904600,	    0.790574,0.821039,
		-0.018082,0.035850,0.165948,	    0.000000,-0.919176,0.393846,	    0.435988,0.869248,
		-0.018082,0.035850,0.165948,	    0.000000,-0.780868,0.624696,	    0.435988,0.869248,
		-0.024186,0.044964,0.187219,	    0.000000,-0.919176,0.393846,	    0.406639,0.873832,
		-0.024186,0.044964,0.187219,	    0.000000,0.426261,0.904600,	    0.781446,0.821039,
		-0.024186,0.044964,0.187219,	    1.000000,-0.000000,0.000000,	    1.716546,0.729020,
		-0.024186,0.044964,0.187219,	    -1.000000,0.000000,0.000000,	    0.924120,0.844963,
		-0.024186,0.035850,0.165948,	    0.000000,-0.919176,0.393846,	    0.435661,0.874388,
		-0.024186,0.035850,0.165948,	    1.000000,-0.000000,0.000000,	    1.680049,0.741986,
		-0.024186,0.035850,0.165948,	    0.988745,0.105793,0.105793,	    1.680049,0.741986,
		-0.024186,0.035850,0.165948,	    0.000000,-0.780868,0.624696,	    0.435661,0.874388,
		0.024341,0.035850,0.165948,	    0.000000,-0.919176,0.393846,	    0.438258,0.833523,
		0.024341,0.035850,0.165948,	    -0.986899,0.114085,0.114085,	    1.736387,0.741986,
		0.024341,0.035850,0.165948,	    -1.000000,0.000000,0.000000,	    1.736387,0.741986,
		0.024341,0.035850,0.165948,	    0.000000,-0.780868,0.624696,	    0.438258,0.833523,
		0.024341,0.044964,0.187219,	    0.000000,-0.919176,0.393846,	    0.409237,0.832966,
		0.024341,0.044964,0.187219,	    -0.000000,0.426261,0.904600,	    2.068637,0.668778,
		0.024341,0.044964,0.187219,	    -1.000000,0.000000,0.000000,	    1.772884,0.729020,
		0.024341,0.044964,0.187219,	    1.000000,0.000000,0.000000,	    0.922880,0.851467,
		0.018237,0.058146,0.181007,	    -0.000000,0.426261,0.904600,	    0.844890,0.816425,
		0.018237,0.058146,0.181007,	    -0.000000,0.186170,0.982518,	    0.844890,0.816425,
		-0.018082,0.058146,0.181007,	    0.000000,0.426261,0.904600,	    0.790574,0.816425,
		-0.018082,0.058146,0.181007,	    -0.000000,0.186170,0.982518,	    0.790574,0.816425,
		-0.013278,0.100294,0.174537,	    -0.000000,0.990681,0.136201,	    0.587438,0.785002,
		-0.013278,0.100294,0.174537,	    -0.000000,0.138351,0.990383,	    0.797759,0.801673,
		0.013712,0.100294,0.174537,	    -0.000000,0.990681,0.136201,	    0.623074,0.785002,
		0.013712,0.100294,0.174537,	    0.000000,0.138351,0.990383,	    0.838122,0.801673,
		-0.024186,0.058146,0.181007,	    0.000000,0.426261,0.904600,	    0.781446,0.816425,
		-0.024186,0.058146,0.181007,	    -1.000000,0.000000,0.000000,	    0.915143,0.829892,
		-0.024186,0.058146,0.181007,	    0.000000,0.186170,0.982518,	    0.781446,0.816425,
		-0.024186,0.058146,0.181007,	    -0.958012,0.286069,0.019450,	    0.915143,0.829892,
		-0.017815,0.100294,0.174537,	    0.000000,0.990681,0.136201,	    0.581449,0.785002,
		-0.017815,0.100294,0.174537,	    0.000000,0.138351,0.990383,	    0.790975,0.801673,
		-0.017815,0.100294,0.174537,	    -0.995216,0.097068,0.011048,	    0.905792,0.781706,
		-0.017815,0.100294,0.174537,	    -0.995315,0.095783,0.013168,	    0.905792,0.781706,
		0.024341,0.058146,0.181007,	    -0.000000,0.426261,0.904600,	    2.068637,0.649294,
		0.024341,0.058146,0.181007,	    0.000000,0.426261,0.904600,	    0.854018,0.816425,
		0.024341,0.058146,0.181007,	    1.000000,0.000000,0.000000,	    0.914088,0.836589,
		0.024341,0.058146,0.181007,	    -0.000000,0.186170,0.982518,	    0.854018,0.816425,
		0.024341,0.058146,0.181007,	    0.957864,0.286559,0.019483,	    0.914088,0.836589,
		0.018247,0.100294,0.174537,	    -0.000000,0.990681,0.136201,	    0.629063,0.785002,
		0.018247,0.100294,0.174537,	    -0.000000,0.138351,0.990383,	    0.844905,0.801673,
		0.018247,0.100294,0.174537,	    0.996178,0.086532,0.011897,	    0.904930,0.789021,
		0.018247,0.100294,0.174537,	    0.996098,0.087693,0.009981,	    0.904930,0.789021,
		0.016660,-0.102150,0.189052,	    -0.000000,0.246767,0.969075,	    0.579069,0.815580,
		0.016660,-0.102150,0.189052,	    0.000000,-0.073370,0.997305,	    0.579069,0.815580,
		0.016660,-0.102150,0.189052,	    1.000000,-0.000000,0.000000,	    0.780126,0.835953,
		0.016660,-0.102150,0.189052,	    0.707105,0.174491,0.685241,	    0.948103,0.701454,
		0.016660,-0.102150,0.189052,	    0.707107,-0.707107,0.000000,	    0.948103,0.701454,
		-0.016660,-0.102150,0.189052,	    0.000000,0.246767,0.969075,	    0.529035,0.815580,
		-0.016660,-0.102150,0.189052,	    0.000000,-0.073370,0.997305,	    0.529035,0.815580,
		-0.016660,-0.102150,0.189052,	    -1.000000,0.000000,0.000000,	    0.529035,0.815580,
		-0.016660,-0.102150,0.189052,	    -0.707105,0.174491,0.685241,	    0.032418,0.695725,
		-0.016660,-0.102150,0.189052,	    -0.707107,-0.707107,0.000000,	    0.032418,0.695725,
		0.016660,-0.102150,0.101052,	    1.000000,0.000000,0.000000,	    0.704588,0.835953,
		0.016660,-0.102150,0.101052,	    -0.000000,-0.189601,-0.981861,	    0.704588,0.835953,
		0.016660,-0.102150,0.101052,	    0.000000,0.518828,-0.854878,	    0.704588,0.835953,
		0.016660,-0.102150,0.101052,	    0.707103,-0.134069,-0.694284,	    0.664142,0.701454,
		0.016660,-0.102150,0.101052,	    0.707107,-0.707107,0.000000,	    0.664142,0.701454,
		-0.016660,-0.102150,0.101052,	    0.000000,-0.189601,-0.981861,	    0.781719,0.835953,
		-0.016660,-0.102150,0.101052,	    0.000000,0.518828,-0.854878,	    0.781719,0.835953,
		-0.016660,-0.102150,0.101052,	    -1.000000,0.000000,0.000000,	    0.414254,0.815580,
		-0.016660,-0.102150,0.101052,	    -0.707103,-0.134069,-0.694284,	    0.293072,0.695725,
		-0.016660,-0.102150,0.101052,	    -0.707107,-0.707107,0.000000,	    0.293072,0.695725,
		-0.015501,0.069850,0.178790,	    0.000000,0.186170,0.982518,	    0.794434,0.812328,
		-0.015501,0.069850,0.178790,	    -0.000000,0.138351,0.990383,	    0.794434,0.812328,
		0.015650,0.069850,0.178790,	    -0.000000,0.186170,0.982518,	    0.841020,0.812328,
		0.015650,0.069850,0.178790,	    -0.000000,0.138351,0.990383,	    0.841020,0.812328,
		-0.020737,0.069850,0.178790,	    -0.952475,0.304434,0.010573,	    0.911938,0.816512,
		-0.020737,0.069850,0.178790,	    0.000000,0.186170,0.982518,	    0.786605,0.812328,
		-0.020737,0.069850,0.178790,	    0.000000,0.138351,0.990383,	    0.786605,0.812328,
		-0.020737,0.069850,0.178790,	    -0.995216,0.097068,0.011048,	    0.911938,0.816512,
		-0.020737,0.069850,0.178790,	    -0.958012,0.286069,0.019450,	    0.911938,0.816512,
		0.020885,0.069850,0.178790,	    -0.000000,0.186170,0.982518,	    0.848850,0.812328,
		0.020885,0.069850,0.178790,	    -0.000000,0.138351,0.990383,	    0.848850,0.812328,
		0.020885,0.069850,0.178790,	    0.996098,0.087693,0.009981,	    0.910950,0.823381,
		0.020885,0.069850,0.178790,	    0.957864,0.286559,0.019483,	    0.910950,0.823381,
		0.020885,0.069850,0.178790,	    0.951622,0.307122,0.009536,	    0.910950,0.823381,
		-0.020990,0.069850,0.155948,	    -0.952475,0.304434,0.010573,	    0.878929,0.816512,
		-0.020990,0.069850,0.155948,	    -0.995216,0.097068,0.011048,	    0.878929,0.816512,
		-0.020990,0.069850,0.155948,	    -0.995315,0.095783,0.013168,	    0.878929,0.816512,
		-0.020990,0.069850,0.155948,	    -0.952528,0.304451,0.000000,	    0.878929,0.816512,
		-0.020990,0.069850,0.155948,	    -0.995401,0.095792,0.000000,	    0.878929,0.816512,
		0.021114,0.069850,0.155948,	    0.996178,0.086532,0.011897,	    0.878621,0.823381,
		0.021114,0.069850,0.155948,	    0.996098,0.087693,0.009981,	    0.878621,0.823381,
		0.021114,0.069850,0.155948,	    0.951622,0.307122,0.009536,	    0.878621,0.823381,
		0.021114,0.069850,0.155948,	    0.951666,0.307136,0.000000,	    0.878621,0.823381,
		0.021114,0.069850,0.155948,	    0.996249,0.086538,-0.000000,	    0.878621,0.823381,
		-0.020990,0.069850,0.075948,	    -0.952528,0.304451,0.000000,	    0.763319,0.816512,
		-0.020990,0.069850,0.075948,	    -0.106803,0.994280,0.000000,	    1.226103,0.277173,
		-0.020990,0.069850,0.075948,	    -0.995401,0.095792,0.000000,	    0.763319,0.816512,
		-0.020990,0.069850,0.075948,	    0.000000,0.000000,-1.000000,	    1.761495,0.513870,
		0.021114,0.069850,0.075948,	    0.951666,0.307136,-0.000000,	    0.765394,0.823381,
		0.021114,0.069850,0.075948,	    0.082322,0.996606,-0.000000,	    1.308263,0.277173,
		0.021114,0.069850,0.075948,	    0.996249,0.086538,-0.000000,	    0.765394,0.823381,
		0.021114,0.069850,0.075948,	    0.000000,0.000000,-1.000000,	    1.829209,0.513870,
		0.015818,0.069850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.766280,0.806125,
		-0.015694,0.069850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.719154,0.806125,
		-0.020990,0.069850,-0.189052,	    -0.952528,0.304451,0.000000,	    0.380359,0.816512,
		-0.020990,0.069850,-0.189052,	    -0.106803,0.994280,0.000000,	    1.226103,0.668742,
		-0.020990,0.069850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.711234,0.806125,
		0.021114,0.069850,-0.189052,	    0.951666,0.307136,0.000000,	    0.390331,0.823381,
		0.021114,0.069850,-0.189052,	    0.082322,0.996606,-0.000000,	    1.308263,0.668742,
		0.021114,0.069850,-0.189052,	    0.000000,0.000000,-1.000000,	    0.774200,0.806125,
		0.011042,0.101170,0.075948,	    0.000000,1.000000,-0.000000,	    0.477539,0.767177,
		0.011042,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    0.627176,0.772298,
		0.011042,0.101170,-0.189052,	    0.000000,1.000000,0.000000,	    0.833650,0.774009,
		0.011042,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.759138,0.795005,
		0.011042,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    1.961717,0.558839,
		-0.011554,0.101170,-0.189052,	    0.000000,1.000000,0.000000,	    0.832375,0.794076,
		-0.011554,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.725346,0.795005,
		-0.011554,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    1.923637,0.558839,
		-0.011554,0.101170,0.075948,	    0.000000,1.000000,-0.000000,	    0.476264,0.787243,
		-0.011554,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    0.594927,0.772298,
		-0.015352,0.101170,-0.189052,	    0.000000,1.000000,0.000000,	    0.832160,0.797448,
		-0.015352,0.101170,-0.189052,	    -0.996342,0.085452,0.000000,	    0.455370,0.831303,
		-0.015352,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.719667,0.795005,
		-0.015352,0.101170,0.075948,	    0.000000,1.000000,-0.000000,	    0.476049,0.790616,
		-0.015352,0.101170,0.075948,	    -0.996342,0.085452,0.000000,	    0.862602,0.831303,
		-0.015352,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    0.589508,0.772298,
		-0.015352,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    1.770563,0.462946,
		0.014839,0.101170,0.075948,	    0.000000,1.000000,-0.000000,	    0.477753,0.763804,
		0.014839,0.101170,0.075948,	    0.997017,0.077178,-0.000000,	    0.862602,0.785106,
		0.014839,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    0.632595,0.772298,
		0.014839,0.101170,0.075948,	    0.000000,0.000000,-1.000000,	    1.819118,0.462946,
		0.014839,0.101170,-0.189052,	    0.000000,1.000000,0.000000,	    0.833864,0.770637,
		0.014839,0.101170,-0.189052,	    0.997017,0.077178,0.000000,	    0.455370,0.785106,
		0.014839,0.101170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.764817,0.795005,
		0.012805,0.070170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.761775,0.806011,
		-0.013577,0.070170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.722321,0.806011,
		-0.018010,0.070170,-0.189052,	    -0.996342,0.085452,0.000000,	    0.455370,0.875300,
		-0.018010,0.070170,-0.189052,	    -0.106803,0.994280,0.000000,	    1.237840,0.668742,
		-0.018010,0.070170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.715691,0.806011,
		-0.018010,0.070170,0.075948,	    -0.996342,0.085452,0.000000,	    0.862602,0.875300,
		-0.018010,0.070170,0.075948,	    -0.106803,0.994280,0.000000,	    1.237840,0.277173,
		-0.018010,0.070170,0.075948,	    0.000000,0.000000,-1.000000,	    1.766287,0.513349,
		0.017239,0.070170,0.075948,	    0.997017,0.077178,-0.000000,	    0.862602,0.829103,
		0.017239,0.070170,0.075948,	    0.082322,0.996606,-0.000000,	    1.296526,0.277173,
		0.017239,0.070170,0.075948,	    0.000000,0.000000,-1.000000,	    1.822977,0.513349,
		0.017239,0.070170,-0.189052,	    0.997017,0.077178,0.000000,	    0.455370,0.829103,
		0.017239,0.070170,-0.189052,	    0.082322,0.996606,-0.000000,	    1.296526,0.668742,
		0.017239,0.070170,-0.189052,	    0.000000,0.000000,-1.000000,	    0.768406,0.806011,
		0.009489,0.071389,-0.189052,	    -0.998616,-0.052600,0.000000,	    0.906404,0.841122,
		0.009489,0.071389,-0.189052,	    0.000000,1.000000,0.000000,	    0.906404,0.841122,
		0.009489,0.071389,-0.189052,	    0.000000,0.000000,-1.000000,	    0.756815,0.805578,
		-0.010298,0.071389,-0.189052,	    0.998180,-0.060302,0.000000,	    0.868796,0.826348,
		-0.010298,0.071389,-0.189052,	    0.000000,1.000000,0.000000,	    0.868796,0.826348,
		-0.010298,0.071389,-0.189052,	    0.000000,0.000000,-1.000000,	    0.727225,0.805578,
		-0.008781,0.096499,-0.189052,	    0.998180,-0.060302,0.000000,	    0.867900,0.840444,
		-0.008781,0.096499,-0.189052,	    0.000000,-1.000000,0.000000,	    0.867900,0.840444,
		-0.008781,0.096499,-0.189052,	    0.000000,0.000000,-1.000000,	    0.729493,0.796663,
		-0.008781,0.096499,-0.189052,	    0.000000,0.000000,-1.000000,	    1.928311,0.565842,
		0.008166,0.096499,-0.189052,	    0.000000,-1.000000,0.000000,	    0.870378,0.840431,
		0.008166,0.096499,-0.189052,	    -0.998616,-0.052600,0.000000,	    0.870378,0.840431,
		0.008166,0.096499,-0.189052,	    0.000000,0.000000,-1.000000,	    1.956871,0.565842,
		0.008166,0.096499,-0.189052,	    0.000000,0.000000,-1.000000,	    0.754837,0.796663,
		0.009489,0.071389,0.148948,	    -0.998616,-0.052600,0.000000,	    0.839335,0.862763,
		0.009489,0.071389,0.148948,	    0.000000,1.000000,-0.000000,	    0.411474,0.797755,
		0.009489,0.071389,0.148948,	    0.000000,0.000000,-1.000000,	    1.829243,0.711074,
		-0.010298,0.071389,0.148948,	    0.998180,-0.060302,0.000000,	    0.905567,0.827067,
		-0.010298,0.071389,0.148948,	    0.000000,1.000000,-0.000000,	    0.410216,0.817549,
		-0.010298,0.071389,0.148948,	    0.000000,0.000000,-1.000000,	    1.789630,0.711074,
		-0.008781,0.096499,0.148948,	    0.998180,-0.060302,0.000000,	    0.905567,0.793602,
		-0.008781,0.096499,0.148948,	    0.000000,-1.000000,0.000000,	    0.409320,0.831645,
		-0.008781,0.096499,0.148948,	    0.000000,0.000000,-1.000000,	    1.792667,0.671461,
		0.008166,0.096499,0.148948,	    0.000000,-1.000000,0.000000,	    0.410133,0.818849,
		0.008166,0.096499,0.148948,	    -0.998616,-0.052600,0.000000,	    0.839335,0.829298,
		0.008166,0.096499,0.148948,	    0.000000,0.000000,-1.000000,	    1.826595,0.671461,
		0.022261,0.027850,-0.171052,	    0.993385,-0.114835,0.000000,	    0.415807,0.870781,
		0.022261,0.027850,-0.171052,	    0.000000,-1.000000,0.000000,	    0.897945,0.844097,
		0.016660,0.027850,-0.171052,	    0.000000,-1.000000,0.000000,	    0.897645,0.848813,
		-0.016660,0.027850,-0.171052,	    0.000000,-1.000000,0.000000,	    0.895862,0.876873,
		-0.022261,0.027850,-0.171052,	    -0.994325,-0.106390,0.000000,	    0.406372,0.864528,
		-0.022261,0.027850,-0.171052,	    0.000000,-1.000000,0.000000,	    0.895562,0.881589,
		0.024341,0.045850,-0.171052,	    1.000000,0.000000,0.000000,	    0.415807,0.850467,
		0.024341,0.045850,-0.171052,	    0.993385,-0.114835,0.000000,	    0.415807,0.850467,
		-0.024186,0.045850,-0.171052,	    -1.000000,0.000000,0.000000,	    0.406372,0.843950,
		-0.024186,0.045850,-0.171052,	    -0.994325,-0.106390,0.000000,	    0.406372,0.843950,
		0.024341,0.059850,-0.171052,	    0.951666,0.307136,-0.000000,	    0.415807,0.834666,
		0.024341,0.059850,-0.171052,	    1.000000,0.000000,0.000000,	    0.415807,0.834666,
		-0.024186,0.059850,-0.171052,	    -0.952528,0.304451,0.000000,	    0.406372,0.827944,
		-0.024186,0.059850,-0.171052,	    -1.000000,0.000000,0.000000,	    0.406372,0.827944,
		0.009489,0.071389,-0.171052,	    -0.998616,-0.052600,0.000000,	    0.390352,0.862763,
		0.009489,0.071389,-0.171052,	    -0.998616,-0.052600,0.000000,	    0.907257,0.827704,
		0.009489,0.071389,-0.171052,	    0.000000,1.000000,-0.000000,	    0.907257,0.827704,
		-0.010298,0.071389,-0.171052,	    0.998180,-0.060302,0.000000,	    0.456584,0.827067,
		-0.010298,0.071389,-0.171052,	    0.998180,-0.060302,0.000000,	    0.844374,0.825879,
		-0.010298,0.071389,-0.171052,	    0.000000,1.000000,0.000000,	    0.844374,0.825879,
		0.017239,0.070170,-0.171052,	    0.997017,0.077178,-0.000000,	    0.483031,0.829103,
		0.017239,0.070170,-0.171052,	    0.082322,0.996606,0.000000,	    1.296526,0.642145,
		0.021114,0.069850,-0.171052,	    0.951666,0.307136,0.000000,	    0.415807,0.823381,
		0.021114,0.069850,-0.171052,	    0.082322,0.996606,-0.000000,	    1.308263,0.642145,
		-0.018010,0.070170,-0.171052,	    -0.996342,0.085452,0.000000,	    0.483031,0.875300,
		-0.018010,0.070170,-0.171052,	    -0.106803,0.994280,0.000000,	    1.237840,0.642145,
		-0.020990,0.069850,-0.171052,	    -0.952528,0.304451,0.000000,	    0.406372,0.816512,
		-0.020990,0.069850,-0.171052,	    -0.106803,0.994280,0.000000,	    1.226103,0.642145,
		0.008166,0.096499,-0.171052,	    0.000000,-1.000000,0.000000,	    0.871231,0.827013,
		0.008166,0.096499,-0.171052,	    -0.998616,-0.052600,0.000000,	    0.390352,0.829298,
		0.008166,0.096499,-0.171052,	    -0.998616,-0.052600,0.000000,	    0.871231,0.827013,
		-0.008781,0.096499,-0.171052,	    0.998180,-0.060302,0.000000,	    0.456584,0.793602,
		-0.008781,0.096499,-0.171052,	    0.998180,-0.060302,0.000000,	    0.843478,0.839975,
		-0.008781,0.096499,-0.171052,	    0.000000,-1.000000,0.000000,	    0.843478,0.839975,
		0.014839,0.101170,-0.171052,	    0.000000,1.000000,-0.000000,	    0.809676,0.770173,
		0.014839,0.101170,-0.171052,	    0.997017,0.077178,0.000000,	    0.483031,0.785106,
		-0.015352,0.101170,-0.171052,	    0.000000,1.000000,0.000000,	    0.807972,0.796984,
		-0.015352,0.101170,-0.171052,	    -0.996342,0.085452,0.000000,	    0.483031,0.831303,
		0.011042,0.101170,-0.171052,	    0.000000,1.000000,-0.000000,	    0.809461,0.773545,
		-0.011554,0.101170,-0.171052,	    0.000000,1.000000,0.000000,	    0.808186,0.793612,
		-0.024660,0.019850,0.149730,	    -1.000000,0.000000,0.000000,	    0.224621,0.088460,
		-0.024660,0.019850,0.149730,	    -0.707108,0.174491,0.685238,	    0.224621,0.088460,
		-0.024660,0.019850,0.149730,	    -0.707107,0.707107,0.000000,	    0.224621,0.088460,
		-0.024660,0.019850,0.085641,	    -1.000000,0.000000,0.000000,	    0.410421,0.088460,
		-0.024660,0.019850,0.085641,	    -0.707107,0.707107,0.000000,	    0.410421,0.088460,
		-0.024660,0.019850,0.085641,	    -0.707103,-0.134069,-0.694284,	    0.410421,0.088460,
		-0.024660,-0.094150,0.107655,	    -1.000000,0.000000,0.000000,	    0.272777,0.673446,
		-0.024660,-0.094150,0.107655,	    -0.707103,-0.134069,-0.694285,	    0.272777,0.673446,
		-0.024660,-0.094150,0.107655,	    -0.707107,-0.707107,0.000000,	    0.272777,0.673446,
		-0.024660,-0.094150,0.178759,	    -0.707108,0.174491,0.685238,	    0.067289,0.673446,
		-0.024660,-0.094150,0.178759,	    -0.707105,0.174491,0.685241,	    0.067289,0.673446,
		-0.024660,-0.094150,0.178759,	    -0.707107,-0.707107,0.000000,	    0.067289,0.673446,
		-0.024660,-0.094150,0.178759,	    -1.000000,0.000000,0.000000,	    0.067289,0.673446,
		0.024660,0.019850,0.149730,	    1.000000,-0.000000,0.000000,	    0.772282,0.061380,
		0.024660,0.019850,0.149730,	    0.707107,0.707107,-0.000000,	    0.772282,0.061380,
		0.024660,0.019850,0.149730,	    0.707108,0.174491,0.685238,	    0.772282,0.061380,
		0.024660,-0.094150,0.178759,	    1.000000,0.000000,0.000000,	    0.912552,0.672597,
		0.024660,-0.094150,0.178759,	    0.707108,0.174491,0.685238,	    0.912552,0.672597,
		0.024660,-0.094150,0.178759,	    0.707105,0.174491,0.685241,	    0.912552,0.672597,
		0.024660,-0.094150,0.178759,	    0.707107,-0.707107,0.000000,	    0.912552,0.672597,
		0.024660,-0.094150,0.107655,	    1.000000,-0.000000,0.000000,	    0.686949,0.672597,
		0.024660,-0.094150,0.107655,	    0.707107,-0.707107,0.000000,	    0.686949,0.672597,
		0.024660,-0.094150,0.107655,	    0.707103,-0.134069,-0.694285,	    0.686949,0.672597,
		0.024660,0.019850,0.085641,	    0.707103,-0.134069,-0.694284,	    0.580911,0.061380,
		0.024660,0.019850,0.085641,	    1.000000,-0.000000,0.000000,	    0.580911,0.061380,
		0.024660,0.019850,0.085641,	    0.707107,0.707107,0.000000,	    0.580911,0.061380,
		0.009890,-0.017995,0.015369,	    0.000000,0.104527,-0.994522,	    0.816596,0.865392,
		0.009890,-0.017995,0.015369,	    1.000000,0.000000,0.000000,	    0.515326,0.865238,
		0.009890,-0.017995,0.015369,	    0.000000,-0.156333,-0.987704,	    0.816596,0.865392,
		0.009890,-0.018029,0.022828,	    1.000000,0.000000,0.000000,	    0.525527,0.865250,
		0.009890,-0.018029,0.022828,	    -0.000000,0.153352,0.988172,	    0.892020,0.829103,
		0.009890,-0.018029,0.022828,	    0.000000,0.999447,0.033248,	    0.630295,0.745561,
		-0.009250,-0.017995,0.015369,	    0.000000,0.104527,-0.994522,	    0.793871,0.865392,
		-0.009250,-0.017995,0.015369,	    -1.000000,0.000000,0.000000,	    0.677807,0.865238,
		-0.009250,-0.017995,0.015369,	    0.000000,-0.156333,-0.987704,	    0.793871,0.865392,
		-0.009250,-0.018029,0.022828,	    -1.000000,0.000000,0.000000,	    0.688009,0.865250,
		-0.009250,-0.018029,0.022828,	    -0.000000,0.153352,0.988172,	    0.865388,0.829103,
		-0.009250,-0.018029,0.022828,	    0.000000,0.999447,0.033248,	    0.629482,0.758357,
		0.009890,-0.023962,0.014742,	    0.000000,0.104527,-0.994522,	    0.816596,0.867730,
		0.009890,-0.023962,0.014742,	    1.000000,0.000000,0.000000,	    0.514468,0.867431,
		0.009890,-0.023962,0.014742,	    0.000000,-0.999991,-0.004139,	    0.704109,0.821035,
		0.009890,-0.023996,0.022951,	    1.000000,0.000000,0.000000,	    0.525695,0.867444,
		0.009890,-0.023996,0.022951,	    0.000000,-0.999991,-0.004139,	    0.704109,0.818078,
		0.009890,-0.023996,0.022951,	    0.000000,-0.999391,-0.034904,	    0.704109,0.818078,
		-0.009250,-0.023996,0.022951,	    -1.000000,0.000000,0.000000,	    0.688177,0.867444,
		-0.009250,-0.023996,0.022951,	    0.000000,-0.999991,-0.004139,	    0.681384,0.818078,
		-0.009250,-0.023996,0.022951,	    0.000000,-0.999391,-0.034904,	    0.681384,0.818078,
		-0.009250,-0.023962,0.014742,	    0.000000,0.104527,-0.994522,	    0.793871,0.867730,
		-0.009250,-0.023962,0.014742,	    -1.000000,0.000000,0.000000,	    0.676949,0.867431,
		-0.009250,-0.023962,0.014742,	    0.000000,-0.999991,-0.004139,	    0.681384,0.821035,
		0.009890,-0.024775,0.073750,	    0.000000,-1.000000,0.000000,	    0.704109,0.799777,
		0.009890,-0.024775,0.073750,	    1.000000,0.000000,0.000000,	    0.595172,0.867730,
		0.009890,-0.018775,0.073750,	    1.000000,0.000000,0.000000,	    0.595172,0.865525,
		0.009890,-0.018775,0.073750,	    0.000000,0.129340,-0.991600,	    0.892020,0.875300,
		0.009890,-0.018775,0.073750,	    0.000000,1.000000,-0.000000,	    0.555920,0.744134,
		-0.009250,-0.018775,0.073750,	    -1.000000,0.000000,0.000000,	    0.757654,0.865525,
		-0.009250,-0.018775,0.073750,	    0.000000,0.129340,-0.991600,	    0.865388,0.875300,
		-0.009250,-0.018775,0.073750,	    0.000000,1.000000,-0.000000,	    0.555106,0.756930,
		-0.009250,-0.024775,0.073750,	    0.000000,-1.000000,0.000000,	    0.681384,0.799777,
		-0.009250,-0.024775,0.073750,	    -1.000000,0.000000,0.000000,	    0.757654,0.867730,
		0.009890,-0.024775,0.096250,	    0.000000,-1.000000,0.000000,	    0.704109,0.791671,
		0.009890,-0.024775,0.096250,	    1.000000,0.000000,0.000000,	    0.625945,0.867730,
		0.009890,-0.018775,0.096250,	    1.000000,0.000000,0.000000,	    0.625945,0.865525,
		-0.009250,-0.018775,0.096250,	    -1.000000,0.000000,0.000000,	    0.788427,0.865525,
		-0.009250,-0.024775,0.096250,	    0.000000,-1.000000,0.000000,	    0.681384,0.791671,
		-0.009250,-0.024775,0.096250,	    -1.000000,0.000000,0.000000,	    0.788427,0.867730,
		0.009890,0.017975,0.095500,	    1.000000,0.000000,0.000000,	    0.624919,0.852016,
		0.009890,0.017975,0.073000,	    1.000000,0.000000,0.000000,	    0.594146,0.852016,
		0.009890,0.017975,0.073000,	    0.000000,-0.152059,-0.988371,	    0.892020,0.831303,
		0.009890,0.017975,0.073000,	    0.000000,-0.999315,-0.037012,	    0.594146,0.852016,
		-0.009250,0.017975,0.073000,	    -1.000000,0.000000,0.000000,	    0.756628,0.852016,
		-0.009250,0.017975,0.073000,	    0.000000,-0.152059,-0.988371,	    0.865388,0.831303,
		-0.009250,0.017975,0.073000,	    0.000000,-0.999315,-0.037012,	    0.756628,0.852016,
		-0.009250,0.017975,0.095500,	    -1.000000,0.000000,0.000000,	    0.787401,0.852016,
		0.009890,0.016475,0.013000,	    1.000000,-0.000000,0.000000,	    0.512085,0.852567,
		0.009890,0.016475,0.013000,	    0.000000,-0.813733,-0.581238,	    0.816596,0.851888,
		0.009890,0.016475,0.013000,	    0.000000,1.000000,0.000000,	    0.841593,0.847765,
		0.009890,0.016475,0.013000,	    0.000000,0.000000,-1.000000,	    0.816596,0.851888,
		-0.009250,0.016475,0.013000,	    -1.000000,0.000000,0.000000,	    0.674567,0.852567,
		-0.009250,0.016475,0.013000,	    0.000000,-0.813733,-0.581238,	    0.793871,0.851888,
		-0.009250,0.016475,0.013000,	    0.000000,1.000000,0.000000,	    0.818869,0.847765,
		-0.009250,0.016475,0.013000,	    0.000000,0.000000,-1.000000,	    0.793871,0.851888,
		-0.009250,0.016475,0.020500,	    -1.000000,0.000000,0.000000,	    0.684824,0.852567,
		-0.009250,0.016475,0.020500,	    0.000000,1.000000,0.000000,	    0.684824,0.852567,
		-0.009250,0.016475,0.020500,	    0.000000,0.000000,1.000000,	    0.865388,0.785106,
		-0.009250,0.016475,0.020500,	    0.000000,-0.970142,0.242537,	    0.684824,0.852567,
		0.009890,0.016475,0.020500,	    1.000000,0.000000,0.000000,	    0.522343,0.852567,
		0.009890,0.016475,0.020500,	    0.000000,1.000000,0.000000,	    0.522343,0.852567,
		0.009890,0.016475,0.020500,	    0.000000,0.000000,1.000000,	    0.892020,0.785106,
		0.009890,0.016475,0.020500,	    0.000000,-0.970142,0.242537,	    0.522343,0.852567,
		0.009890,0.023975,0.002500,	    1.000000,-0.000000,0.000000,	    0.497724,0.849811,
		0.009890,0.023975,0.002500,	    0.000000,-0.813733,-0.581238,	    0.816596,0.848950,
		0.009890,0.023975,0.002500,	    0.000000,-0.902862,-0.429931,	    0.816596,0.848950,
		-0.009250,0.023975,0.002500,	    -1.000000,0.000000,0.000000,	    0.660206,0.849811,
		-0.009250,0.023975,0.002500,	    0.000000,-0.813733,-0.581238,	    0.793871,0.848950,
		-0.009250,0.023975,0.002500,	    0.000000,-0.902862,-0.429931,	    0.793871,0.848950,
		0.009890,-0.001525,0.098500,	    1.000000,0.000000,0.000000,	    0.629022,0.859184,
		-0.009250,-0.001525,0.098500,	    -1.000000,0.000000,0.000000,	    0.791504,0.859184,
		-0.009250,-0.001525,0.076000,	    -1.000000,0.000000,0.000000,	    0.760731,0.859184,
		-0.009250,-0.001525,0.076000,	    0.000000,-0.152059,-0.988371,	    0.865388,0.854648,
		-0.009250,-0.001525,0.076000,	    0.000000,0.129340,-0.991600,	    0.865388,0.854648,
		0.009890,-0.001525,0.076000,	    0.000000,-0.152059,-0.988371,	    0.892020,0.854648,
		0.009890,-0.001525,0.076000,	    0.000000,0.129340,-0.991600,	    0.892020,0.854648,
		0.009890,-0.001525,0.076000,	    1.000000,-0.000000,0.000000,	    0.598249,0.859184,
		0.009890,-0.003025,0.020500,	    0.000000,0.000000,1.000000,	    0.892020,0.809971,
		0.009890,-0.003025,0.020500,	    -0.000000,0.153352,0.988172,	    0.892020,0.809971,
		0.009890,-0.003025,0.020500,	    1.000000,-0.000000,0.000000,	    0.522343,0.859735,
		-0.009250,-0.003025,0.020500,	    0.000000,0.000000,1.000000,	    0.865388,0.809971,
		-0.009250,-0.003025,0.020500,	    0.000000,0.153352,0.988172,	    0.865388,0.809971,
		-0.009250,-0.003025,0.020500,	    -1.000000,0.000000,0.000000,	    0.684824,0.859735,
		-0.009250,-0.003025,0.013000,	    -1.000000,0.000000,0.000000,	    0.674567,0.859735,
		-0.009250,-0.003025,0.013000,	    0.000000,0.000000,-1.000000,	    0.793871,0.859527,
		-0.009250,-0.003025,0.013000,	    0.000000,-0.156333,-0.987704,	    0.793871,0.859527,
		0.009890,-0.003025,0.013000,	    0.000000,0.000000,-1.000000,	    0.816596,0.859527,
		0.009890,-0.003025,0.013000,	    0.000000,-0.156333,-0.987704,	    0.816596,0.859527,
		0.009890,-0.003025,0.013000,	    1.000000,0.000000,0.000000,	    0.512085,0.859735,
		0.009890,-0.024775,0.045250,	    0.000000,-0.999391,-0.034904,	    0.704109,0.810044,
		0.009890,-0.024775,0.045250,	    0.000000,-1.000000,0.000000,	    0.704109,0.810044,
		0.009890,-0.024775,0.045250,	    1.000000,-0.000000,0.000000,	    0.556193,0.867730,
		-0.009250,-0.024775,0.045250,	    0.000000,-0.999391,-0.034904,	    0.681384,0.810044,
		-0.009250,-0.024775,0.045250,	    0.000000,-1.000000,0.000000,	    0.681384,0.810044,
		-0.009250,-0.024775,0.045250,	    -1.000000,0.000000,0.000000,	    0.718675,0.867730,
		-0.009250,-0.018775,0.045250,	    -1.000000,0.000000,0.000000,	    0.718675,0.865525,
		-0.009250,-0.018775,0.045250,	    0.000000,0.999447,0.033248,	    0.596733,0.757729,
		-0.009250,-0.018775,0.045250,	    0.000000,1.000000,0.000000,	    0.596733,0.757729,
		0.009890,-0.018775,0.045250,	    -0.000000,0.999447,0.033248,	    0.597546,0.744933,
		0.009890,-0.018775,0.045250,	    0.000000,1.000000,-0.000000,	    0.597546,0.744933,
		0.009890,-0.018775,0.045250,	    1.000000,-0.000000,0.000000,	    0.556193,0.865525,
		0.009890,0.019475,0.032500,	    0.000000,-0.970142,0.242537,	    0.841593,0.863305,
		0.009890,0.019475,0.032500,	    0.000000,-0.999315,-0.037012,	    0.841593,0.863305,
		0.009890,0.019475,0.032500,	    1.000000,-0.000000,0.000000,	    0.538755,0.851465,
		-0.009250,0.019475,0.032500,	    -1.000000,0.000000,0.000000,	    0.701237,0.851465,
		-0.009250,0.019475,0.032500,	    0.000000,-0.970142,0.242537,	    0.818869,0.863305,
		-0.009250,0.019475,0.032500,	    0.000000,-0.999315,-0.037012,	    0.818869,0.863305,
		0.007714,-0.008539,0.052781,	    1.000000,-0.000000,0.000000,	    0.643888,0.783544,
		0.007714,-0.008539,0.052781,	    0.000000,0.531165,-0.847269,	    0.691610,0.783544,
		0.007714,-0.008539,0.052781,	    0.000000,-0.876438,-0.481515,	    0.691610,0.783544,
		0.007714,-0.010658,0.056638,	    0.000000,-0.876438,-0.481515,	    0.691610,0.784356,
		0.007714,-0.010658,0.056638,	    1.000000,0.000000,0.000000,	    0.649843,0.784356,
		0.007714,-0.010658,0.056638,	    0.000000,-0.623121,0.782126,	    0.741604,0.784356,
		-0.007215,-0.008539,0.052781,	    0.000000,-0.876438,-0.481515,	    0.668885,0.783544,
		-0.007215,-0.008539,0.052781,	    0.000000,0.531165,-0.847269,	    0.668885,0.783544,
		-0.007215,-0.008539,0.052781,	    -1.000000,0.000000,0.000000,	    0.693883,0.783544,
		-0.007215,-0.010658,0.056638,	    0.000000,-0.876438,-0.481515,	    0.668885,0.784356,
		-0.007215,-0.010658,0.056638,	    -1.000000,0.000000,0.000000,	    0.699837,0.784356,
		-0.007215,-0.010658,0.056638,	    0.000000,-0.623121,0.782126,	    0.718880,0.784356,
		0.007714,0.022475,0.059000,	    1.000000,-0.000000,0.000000,	    0.653489,0.771651,
		0.007714,0.022475,0.059000,	    0.000000,-0.051214,-0.998688,	    0.691610,0.771651,
		-0.007215,0.022475,0.059000,	    0.000000,-0.051214,-0.998688,	    0.668885,0.771651,
		-0.007215,0.022475,0.059000,	    -1.000000,0.000000,0.000000,	    0.703484,0.771651,
		-0.007215,0.022475,0.066500,	    -1.000000,0.000000,0.000000,	    0.715063,0.771651,
		-0.007215,0.022475,0.066500,	    0.000000,0.051214,0.998688,	    0.718880,0.771651,
		0.007714,0.022475,0.066500,	    1.000000,-0.000000,0.000000,	    0.665069,0.771651,
		0.007714,0.022475,0.066500,	    -0.000000,0.051214,0.998688,	    0.741604,0.771651,
		0.007714,0.002975,0.067500,	    1.000000,0.000000,0.000000,	    0.666613,0.779128,
		0.007714,0.002975,0.067500,	    0.000000,-0.623121,0.782126,	    0.741604,0.779128,
		0.007714,0.002975,0.067500,	    -0.000000,0.051214,0.998688,	    0.741604,0.779128,
		-0.007215,0.002975,0.067500,	    -1.000000,0.000000,0.000000,	    0.716607,0.779128,
		-0.007215,0.002975,0.067500,	    0.000000,-0.623121,0.782126,	    0.718880,0.779128,
		-0.007215,0.002975,0.067500,	    -0.000000,0.051214,0.998688,	    0.718880,0.779128,
		-0.007215,0.002975,0.060000,	    0.000000,0.531165,-0.847269,	    0.668885,0.779128,
		-0.007215,0.002975,0.060000,	    0.000000,-0.051214,-0.998688,	    0.668885,0.779128,
		-0.007215,0.002975,0.060000,	    -1.000000,0.000000,0.000000,	    0.705028,0.779128,
		0.007714,0.002975,0.060000,	    1.000000,0.000000,0.000000,	    0.655033,0.779128,
		0.007714,0.002975,0.060000,	    0.000000,0.531165,-0.847269,	    0.691610,0.779128,
		0.007714,0.002975,0.060000,	    0.000000,-0.051214,-0.998688,	    0.691610,0.779128,
		0.003073,0.102150,0.155000,	    0.000000,-1.000000,0.000000,	    0.556398,0.821674,
		0.003073,0.102150,0.155000,	    -1.000000,0.000000,0.000000,	    0.741569,0.841069,
		0.003073,0.102150,0.155000,	    -0.000000,0.132162,0.991228,	    0.818869,0.841069,
		0.003073,0.102150,0.129000,	    0.000000,-1.000000,0.000000,	    0.556398,0.831054,
		0.003073,0.102150,0.129000,	    -1.000000,0.000000,0.000000,	    0.706417,0.841069,
		0.003073,0.102150,0.129000,	    0.000000,0.447213,-0.894428,	    0.643888,0.831054,
		0.013303,0.102150,0.129000,	    0.000000,-1.000000,0.000000,	    0.579123,0.831054,
		0.013303,0.102150,0.129000,	    0.000000,0.447213,-0.894428,	    0.666613,0.831054,
		0.013303,0.102150,0.129000,	    1.000000,0.000000,0.000000,	    0.743913,0.841069,
		0.013303,0.102150,0.155000,	    0.000000,-1.000000,0.000000,	    0.579123,0.821674,
		0.013303,0.102150,0.155000,	    1.000000,0.000000,0.000000,	    0.779065,0.841069,
		0.013303,0.102150,0.155000,	    -0.000000,0.132162,0.991228,	    0.841593,0.841069,
		0.003073,0.117150,0.153000,	    -1.000000,0.000000,0.000000,	    0.738865,0.835005,
		0.003073,0.117150,0.153000,	    -0.000000,0.132162,0.991228,	    0.818869,0.835005,
		0.003073,0.117150,0.153000,	    0.000000,0.882352,-0.470590,	    0.643888,0.821674,
		0.003073,0.105150,0.130500,	    -1.000000,0.000000,0.000000,	    0.708445,0.839857,
		0.003073,0.105150,0.130500,	    0.000000,0.447213,-0.894428,	    0.643888,0.830468,
		0.003073,0.105150,0.130500,	    0.000000,0.882352,-0.470590,	    0.643888,0.830468,
		0.013303,0.105150,0.130500,	    0.000000,0.447213,-0.894428,	    0.666613,0.830468,
		0.013303,0.105150,0.130500,	    1.000000,0.000000,0.000000,	    0.745940,0.839857,
		0.013303,0.105150,0.130500,	    0.000000,0.882352,-0.470590,	    0.666613,0.830468,
		0.013303,0.117150,0.153000,	    1.000000,0.000000,0.000000,	    0.776361,0.835005,
		0.013303,0.117150,0.153000,	    0.000000,0.132162,0.991228,	    0.841593,0.835005,
		0.013303,0.117150,0.153000,	    0.000000,0.882352,-0.470590,	    0.666613,0.821674,
		0.002696,0.100350,-0.182800,	    0.000000,-1.000000,0.000000,	    0.906779,0.875317,
		0.002696,0.100350,-0.182800,	    1.000000,0.000000,0.000000,	    1.817799,0.753367,
		0.002696,0.100350,-0.182800,	    0.000000,0.132170,-0.991227,	    0.906779,0.875317,
		0.002696,0.100350,-0.156800,	    0.000000,-1.000000,0.000000,	    0.906779,0.842835,
		0.002696,0.100350,-0.156800,	    1.000000,0.000000,0.000000,	    1.857412,0.753367,
		0.002696,0.100350,-0.156800,	    -0.000000,0.447185,0.894441,	    0.906779,0.842835,
		-0.002696,0.100350,-0.156800,	    0.000000,-1.000000,0.000000,	    0.894573,0.842835,
		-0.002696,0.100350,-0.156800,	    -0.000000,0.447185,0.894441,	    0.894573,0.842835,
		-0.002696,0.100350,-0.156800,	    -1.000000,0.000000,0.000000,	    1.815158,0.753367,
		-0.002696,0.100350,-0.182800,	    0.000000,-1.000000,0.000000,	    0.894573,0.875317,
		-0.002696,0.100350,-0.182800,	    -1.000000,0.000000,0.000000,	    1.775546,0.753367,
		-0.002696,0.100350,-0.182800,	    0.000000,0.132170,-0.991227,	    0.894573,0.875317,
		0.002696,0.115350,-0.180800,	    1.000000,0.000000,0.000000,	    1.820847,0.727759,
		0.002696,0.115350,-0.180800,	    0.000000,0.132170,-0.991227,	    0.906779,0.808154,
		0.002696,0.115350,-0.180800,	    0.000000,0.882354,0.470587,	    0.906779,0.785056,
		0.002696,0.103350,-0.158300,	    1.000000,0.000000,0.000000,	    1.855127,0.748246,
		0.002696,0.103350,-0.158300,	    -0.000000,0.447185,0.894441,	    0.906779,0.801854,
		0.002696,0.103350,-0.158300,	    0.000000,0.882354,0.470587,	    0.906779,0.801854,
		-0.002696,0.103350,-0.158300,	    0.000000,0.447185,0.894441,	    0.894573,0.801854,
		-0.002696,0.103350,-0.158300,	    -1.000000,0.000000,0.000000,	    1.812873,0.748246,
		-0.002696,0.103350,-0.158300,	    -0.000000,0.882354,0.470587,	    0.894573,0.801854,
		-0.002696,0.115350,-0.180800,	    -1.000000,0.000000,0.000000,	    1.778593,0.727759,
		-0.002696,0.115350,-0.180800,	    0.000000,0.132170,-0.991227,	    0.894573,0.808154,
		-0.002696,0.115350,-0.180800,	    0.000000,0.882354,0.470587,	    0.894573,0.785056,
		-0.013302,0.102150,0.155000,	    0.000000,-1.000000,0.000000,	    0.581395,0.821674,
		-0.013302,0.102150,0.155000,	    -1.000000,0.000000,0.000000,	    0.704073,0.841069,
		-0.013302,0.102150,0.155000,	    0.000000,0.132162,0.991228,	    0.693883,0.831064,
		-0.013302,0.102150,0.129000,	    0.000000,-1.000000,0.000000,	    0.581395,0.831054,
		-0.013302,0.102150,0.129000,	    -1.000000,0.000000,0.000000,	    0.668921,0.841069,
		-0.013302,0.102150,0.129000,	    0.000000,0.447213,-0.894428,	    0.668885,0.831064,
		-0.003129,0.102150,0.129000,	    0.000000,-1.000000,0.000000,	    0.604120,0.831054,
		-0.003129,0.102150,0.129000,	    0.000000,0.447213,-0.894428,	    0.691610,0.831064,
		-0.003129,0.102150,0.129000,	    1.000000,0.000000,0.000000,	    0.781408,0.841069,
		-0.003129,0.102150,0.155000,	    0.000000,-1.000000,0.000000,	    0.604120,0.821674,
		-0.003129,0.102150,0.155000,	    1.000000,0.000000,0.000000,	    0.816560,0.841069,
		-0.003129,0.102150,0.155000,	    0.000000,0.132162,0.991228,	    0.716607,0.831064,
		-0.013302,0.117150,0.153000,	    -1.000000,0.000000,0.000000,	    0.701369,0.835005,
		-0.013302,0.117150,0.153000,	    0.000000,0.132162,0.991228,	    0.693883,0.825000,
		-0.013302,0.117150,0.153000,	    0.000000,0.882352,-0.470590,	    0.668885,0.825000,
		-0.013302,0.105150,0.130500,	    -1.000000,0.000000,0.000000,	    0.670949,0.839857,
		-0.013302,0.105150,0.130500,	    0.000000,0.447213,-0.894428,	    0.668885,0.829851,
		-0.013302,0.105150,0.130500,	    0.000000,0.882352,-0.470590,	    0.668885,0.829851,
		-0.003129,0.105150,0.130500,	    0.000000,0.447213,-0.894428,	    0.691610,0.829851,
		-0.003129,0.105150,0.130500,	    1.000000,0.000000,0.000000,	    0.783436,0.839857,
		-0.003129,0.105150,0.130500,	    0.000000,0.882352,-0.470590,	    0.691610,0.829851,
		-0.003129,0.117150,0.153000,	    1.000000,0.000000,0.000000,	    0.813857,0.835005,
		-0.003129,0.117150,0.153000,	    0.000000,0.132162,0.991228,	    0.716607,0.825000,
		-0.003129,0.117150,0.153000,	    0.000000,0.882352,-0.470590,	    0.691610,0.825000
	};
	glGenBuffers(1, &g_vertexBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);

	//インデックスバッファ
	int indices[] =
	{
		116,210,220,
		493,490,487,
		488,513,504,
		488,484,513,
		513,496,502,
		513,502,504,
		485,510,514,
		491,510,485,
		511,497,515,
		511,498,497,
		494,512,492,
		494,507,512,
		512,507,500,
		512,500,499,
		495,505,508,
		495,489,505,
		348,254,246,
		348,349,254,
		248,251,349,
		248,349,348,
		349,259,254,
		349,346,259,
		251,256,346,
		251,346,349,
		344,246,263,
		348,246,344,
		248,348,267,
		267,348,344,
		272,334,257,
		334,347,257,
		275,260,347,
		334,275,347,
		330,264,278,
		330,345,264,
		281,268,345,
		281,345,330,
		327,304,301,
		327,341,304,
		287,290,342,
		287,342,328,
		343,307,305,
		338,307,343,
		294,343,291,
		294,338,343,
		298,308,339,
		324,298,339,
		284,340,295,
		284,325,340,
		322,124,230,
		322,230,336,
		130,336,240,
		130,322,336,
		133,243,332,
		133,332,320,
		320,234,126,
		320,332,234,
		231,276,335,
		337,231,335,
		241,337,273,
		337,335,273,
		333,279,235,
		333,331,279,
		244,282,333,
		333,282,331,
		326,302,299,
		326,329,302,
		285,288,329,
		285,329,326,
		100,125,323,
		318,100,323,
		108,323,131,
		108,318,323,
		316,127,102,
		316,321,127,
		112,134,321,
		112,321,316,
		70,101,319,
		314,70,319,
		86,319,109,
		86,314,319,
		89,113,317,
		89,317,310,
		76,317,103,
		310,317,76,
		313,17,33,
		313,312,17,
		56,54,313,
		54,312,313,
		315,33,71,
		313,33,315,
		56,315,87,
		56,313,315,
		312,77,17,
		312,311,77,
		54,90,312,
		90,311,312,
		48,52,46,
		48,50,52,
		30,14,27,
		30,27,43,
		48,62,50,
		48,66,62,
		33,37,58,
		33,58,71,
		77,72,21,
		17,77,21,
		78,52,82,
		46,52,78,
		143,136,138,
		143,138,141,
		143,141,149,
		149,141,145,
		136,153,157,
		136,157,138,
		70,59,92,
		70,92,101,
		76,103,73,
		73,103,96,
		55,57,106,
		55,106,104,
		88,110,106,
		57,88,106,
		55,104,91,
		91,104,114,
		142,139,161,
		142,161,163,
		142,163,146,
		146,163,169,
		140,158,177,
		139,178,161,
		100,93,117,
		100,117,125,
		102,120,97,
		102,127,120,
		107,129,105,
		107,128,129,
		107,111,128,
		111,132,128,
		105,135,115,
		105,129,135,
		22,137,144,
		38,22,144,
		150,94,147,
		60,95,151,
		61,144,152,
		38,144,61,
		74,154,98,
		155,159,99,
		75,156,137,
		22,75,137,
		165,167,53,
		165,53,51,
		148,170,93,
		93,170,117,
		165,51,173,
		173,51,63,
		160,97,179,
		97,120,179,
		167,182,83,
		167,83,53,
		191,23,39,
		191,186,23,
		44,28,187,
		44,187,192,
		509,506,503,
		509,503,501,
		15,196,29,
		29,196,188,
		201,34,18,
		197,201,18,
		16,31,202,
		16,202,198,
		420,2,481,
		487,490,486,
		45,193,203,
		32,45,203,
		164,162,208,
		164,208,206,
		207,209,166,
		209,168,166,
		164,206,211,
		164,211,171,
		207,174,212,
		207,166,174,
		162,180,215,
		162,215,208,
		209,216,183,
		209,183,168,
		221,213,175,
		222,176,64,
		172,214,118,
		481,2,13,
		225,84,184,
		226,185,217,
		181,121,218,
		122,227,219,
		124,119,223,
		124,223,230,
		232,65,67,
		232,224,65,
		126,234,228,
		126,228,123,
		236,85,229,
		236,79,85,
		128,239,238,
		128,238,129,
		300,303,306,
		300,306,309,
		128,132,242,
		128,242,239,
		271,274,258,
		271,258,252,
		129,238,245,
		129,245,135,
		270,269,283,
		270,249,269,
		247,255,49,
		247,49,47,
		255,261,49,
		261,68,49,
		247,47,265,
		265,47,80,
		239,270,238,
		239,271,270,
		233,262,277,
		233,69,262,
		237,280,266,
		237,266,81,
		239,274,271,
		239,242,274,
		238,283,245,
		238,270,283,
		270,271,286,
		271,289,286,
		271,292,289,
		271,252,292,
		296,293,253,
		296,253,250,
		270,286,297,
		270,297,249,
		424,440,7,
		440,10,7,
		428,6,443,
		443,6,8,
		429,441,425,
		429,444,441,
		424,5,436,
		424,7,5,
		428,432,4,
		428,4,6,
		420,423,3,
		420,3,2,
		445,11,442,
		445,9,11,
		417,0,416,
		417,1,0,
		400,410,414,
		408,400,414,
		409,415,413,
		409,413,405,
		401,412,411,
		401,402,412,
		397,376,388,
		397,382,376,
		394,385,383,
		394,383,398,
		391,389,377,
		391,377,379,
		392,399,390,
		395,399,392,
		430,437,426,
		430,433,437,
		448,423,420,
		448,447,423,
		413,447,448,
		405,413,448,
		449,421,418,
		449,418,451,
		406,450,403,
		403,450,452,
		453,417,416,
		453,416,446,
		402,453,412,
		412,453,446,
		457,438,434,
		457,454,438,
		386,380,455,
		386,455,458,
		459,432,428,
		460,459,428,
		385,459,383,
		383,459,460,
		461,431,427,
		461,427,463,
		384,462,378,
		378,462,464,
		456,424,436,
		465,424,456,
		377,465,456,
		379,377,456,
		469,396,393,
		469,393,466,
		470,400,408,
		470,467,400,
		394,472,385,
		471,472,394,
		409,405,472,
		471,409,472,
		473,381,387,
		473,475,381,
		476,407,404,
		474,407,476,
		391,379,477,
		468,391,477,
		468,402,401,
		468,477,402,
		435,439,482,
		439,478,482,
		422,483,419,
		419,483,479,
		480,5,12,
		436,5,480,
		417,480,1,
		480,12,1,
		432,481,4,
		481,13,4,
		199,19,373,
		356,350,353,
		370,374,363,
		370,363,366,
		359,40,351,
		194,41,360,
		354,42,35,
		354,352,42,
		204,355,36,
		204,357,355,
		205,361,358,
		205,195,361,
		375,20,24,
		375,24,364,
		367,365,25,
		189,368,26,
		200,371,369,
		200,369,190,
		199,373,372,
		356,362,350,
		519,525,516,
		519,522,525,
		517,528,531,
		520,517,531,
		521,532,523,
		523,532,534,
		524,535,526,
		526,535,537,
		518,527,529,
		527,538,529,
		533,530,539,
		533,539,536,
		549,540,543,
		549,543,546,
		541,552,555,
		541,555,544,
		547,545,556,
		547,556,558,
		550,548,559,
		550,559,561,
		542,551,553,
		551,562,553,
		557,554,563,
		560,557,563,
		567,573,564,
		570,573,567,
		565,576,579,
		568,565,579,
		571,569,580,
		571,580,582,
		572,583,574,
		574,583,585,
		575,577,566,
		575,586,577,
		581,578,587,
		584,581,587
	};
	glGenBuffers(1, &g_indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW);

	glGenBuffers(1, &g_indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW);
	//テクスチャーをファイルから作成
	g_texture = CreateTextureFromBMPFile("Grid.bmp");
	//テクスチャーのサンプラーとテクスチャーユニットを紐付ける
	GLuint texture = glGetUniformLocation(g_simpleShader, "g_textureSampler");
	glUniform1i(texture, 0);
}
//
//
//シェーダー読み込みコンパイル
GLuint LoadShaders(const char* vertexFilename, const char* fragmentFilename)
{
	GLuint vertexShader = glCreateShader(GL_VERTEX_SHADER);
	GLuint fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
	//バーテックスシェーダのコードを読み込む
	std::string VertexShaderCode;
	std::ifstream VertexShaderStream(vertexFilename, std::ios::in);
	if (VertexShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << VertexShaderStream.rdbuf();
		VertexShaderCode = sstr.str();
		VertexShaderStream.close();
	}
	else
	{
		printf("ファイルが見つかりません\n");
		return 0;
	}
	//フラグメントシェーダーのコードを読み込む
	std::string FragmentShaderCode;
	std::ifstream FragmentShaderStream(fragmentFilename, std::ios::in);
	if (FragmentShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << FragmentShaderStream.rdbuf();
		FragmentShaderCode = sstr.str();
		FragmentShaderStream.close();
	}
	GLint Result = GL_FALSE;
	int infoLength;
	//バーテックスシェーダをコンパイル
	char const * VertexSourcePointer = VertexShaderCode.c_str();
	glShaderSource(vertexShader, 1, &VertexSourcePointer, NULL);
	glCompileShader(vertexShader);

	glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(vertexShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(vertexShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//フラグメントシェーダーをコンパイル
	char const * FragmentSourcePointer = FragmentShaderCode.c_str();
	glShaderSource(fragmentShader, 1, &FragmentSourcePointer, NULL);
	glCompileShader(fragmentShader);

	glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(fragmentShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(fragmentShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//バーテックスシェーダーとフラグメントシェーダーを「プログラムオブジェクト」に関連付ける
	GLuint program = glCreateProgram();
	glAttachShader(program, vertexShader);
	glAttachShader(program, fragmentShader);
	glLinkProgram(program);

	glGetProgramiv(program, GL_LINK_STATUS, &Result);
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> ProgramErrorMessage(infoLength + 1);
		glGetProgramInfoLog(program, infoLength, NULL, &ProgramErrorMessage[0]);
		printf(&ProgramErrorMessage[0]);
	}

	return program;
}
//
//
//
GLuint CreateTextureFromBMPFile(const char * imagepath)
{
	FILE * fp=fopen(imagepath, "rb");
	if (!fp)
	{
		printf("ファイルを開けません\n");
		return 0;
	}
	//BMPヘッダーは54バイト
	unsigned char header[54];
	if (fread(header, 1, 54, fp) != 54)
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	//ビットマップであれば"BM"ヘッダーがあるはず
	if (header[0] != 'B' || header[1] != 'M')
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	// 24bitカラーかどうか
	if (*(int*)&(header[30]) != 0 || *(int*)&(header[28]) != 24)
	{
		printf("24ビットカラーではありません");
		return 0;
	}
	//ピクセルデータを読み込む
	unsigned int width = *(int*)&(header[18]);
	unsigned int height = *(int*)&(header[22]);
	unsigned int size = *(int*)&(header[34]);

	unsigned char* data = new unsigned char[size];
	fread(data, 1, size, fp);
	//この時点でもはやファイルは不要
	fclose(fp);

	//空（から）のテクスチャーを作成
	GLuint id;
	glGenTextures(1, &id);
	//テクスチャーをバインドして、これからそのテクスチャーにデータを送り込むことを指示する
	glBindTexture(GL_TEXTURE_2D, id);
	// ピクセルデータをテクスチャーに送り込む
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_BGR, GL_UNSIGNED_BYTE, data);
	//この時点でメモリー上のデータは不要になる
	delete data;
	//テクスチャーの仕様を設定
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glGenerateMipmap(GL_TEXTURE_2D);

	return id;
}
//
//
//描画
void Render()
{
    //Depthテスト有効にして陰面消去
	glEnable(GL_DEPTH_TEST);
    //ビューポートをセット
    XWindowAttributes gwa;
    XGetWindowAttributes(g_dpy, g_win, &gwa);
    glViewport(0, 0, gwa.width, gwa.height);
    //クリア
    glClearColor(0.0f, 0.0f, 0.4f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //これ以降使うシェーダーのセット
    glUseProgram(g_simpleShader);
    glBindVertexArray(g_vao);
    //VBOをセット
    glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
    //シェーダーに頂点内の「位置成分」の場所とサイズを教える
	GLint posAttrib = glGetAttribLocation(g_simpleShader, "g_position");
	glEnableVertexAttribArray(posAttrib);
	glVertexAttribPointer(posAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)0);
	//シェーダーに頂点内の「法線成分」の場所とサイズを教える
	GLint normalAttrib = glGetAttribLocation(g_simpleShader, "g_normal");
	glEnableVertexAttribArray(normalAttrib);
	glVertexAttribPointer(normalAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)(sizeof(GLfloat)*3));
	//シェーダーに頂点内の「テクスチャー座標成分」の場所とサイズを教える
	GLint uvAttrib = glGetAttribLocation(g_simpleShader, "g_uv");
	glEnableVertexAttribArray(uvAttrib);
	glVertexAttribPointer(uvAttrib, 2, GL_FLOAT, GL_FALSE, sizeof(GLfloat) * 8, (void*)(sizeof(GLfloat) * 6));

    //行列を渡す
    //ワールド行列
    mat4 world = mat4(1.0);
	static float angle = 0;
	angle+=0.01;
	world=rotate(world,angle,vec3(0,1,0));

	//ビュー行列
	mat4 view = lookAt(vec3(0, 0.3, 0.3), vec3(0, 0, 0), vec3(0, 1, 0));
	//プロジェクション行列
	mat4 proj = perspectiveFovRH(45.0, (double)WINDOW_WIDTH, (double)WINDOW_HEIGHT, 0.1, 10000.0);
	//３つの合成行列をシェーダーに渡す
	GLint matrix = glGetUniformLocation(g_simpleShader, "g_worldViewProjMat");
	mat4 wvp=proj*view*world;
	glUniformMatrix4fv(matrix, 1, GL_FALSE, (GLfloat*)&wvp);
	//法線を回転させるためにワールド行列だけを個別に渡す
	GLint worldMatrix = glGetUniformLocation(g_simpleShader, "g_worldMat");
	glUniformMatrix4fv(worldMatrix, 1, GL_FALSE, (GLfloat*)&(world));
	//シェーダーにライトの方向を渡す
	GLint light = glGetUniformLocation(g_simpleShader, "g_lightVector");
	glUniform4f(light, 0,0,1,0);
	//テクスチャーをセットする
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, g_texture);
    //描画
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indexBuffer);
	glDrawElements(GL_TRIANGLES, 382*3, GL_UNSIGNED_INT, (void*)0);
}
