#include<stdio.h>
#include <string.h>
#include <vector>
#include <fstream>
#include <sstream>
#include<X11/X.h>
#include<X11/Xlib.h>
#include "external/glm/glm.hpp"
//本サンプルに限り、GLMはサンプル付属のものを使う
//その理由は、本サンプルではglm の行列補間であるinterpolateを使っているが
//GLMのバージョンによっては間違った結果を出すことがある不安定な関数であるため
//確実に動くことが検証済みのGLM（サンプルフォル内のexternal）を使う
#include "external/glm/gtc/matrix_transform.hpp"
#include "external/glm/gtx/matrix_interpolation.hpp"
#include "external/glm/gtc/quaternion.hpp"
#include "external/glm/common.hpp"
#include<GL/gl.h>
#include<GL/glx.h>
#include<GL/glu.h>

using namespace glm;

#include<GL/glext.h>//(OpenGL)Eextensionsの定義が詰まっているglext.hをインクルード　ステップ①

//(OpenGL)Eextensions関数のポインターの受け皿を用意　ステップ②
PFNGLCREATEPROGRAMPROC glCreateProgram = NULL;
PFNGLGENVERTEXARRAYSPROC glGenVertexArrays = NULL;
PFNGLBINDVERTEXARRAYPROC glBindVertexArray = NULL;
PFNGLCREATESHADERPROC glCreateShader = NULL;
PFNGLSHADERSOURCEPROC glShaderSource = NULL;
PFNGLCOMPILESHADERPROC glCompileShader = NULL;
PFNGLGETSHADERIVPROC glGetShaderiv = NULL;
PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog = NULL;
PFNGLATTACHSHADERPROC glAttachShader = NULL;
PFNGLLINKPROGRAMPROC glLinkProgram = NULL;
PFNGLGETPROGRAMIVPROC glGetProgramiv = NULL;
PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog = NULL;
PFNGLGENBUFFERSPROC glGenBuffers = NULL;
PFNGLBINDBUFFERPROC glBindBuffer = NULL;
PFNGLBUFFERDATAPROC glBufferData = NULL;
PFNGLUSEPROGRAMPROC glUseProgram = NULL;
PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray = NULL;
PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer = NULL;
PFNGLGETATTRIBLOCATIONPROC glGetAttribLocation = NULL;
PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation = NULL;
PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv = NULL;
PFNGLUNIFORM4FPROC glUniform4f = NULL;
PFNGLUNIFORM1IARBPROC glUniform1i = NULL;
PFNGLVERTEXATTRIBIPOINTERPROC glVertexAttribIPointer = NULL;
PFNGLGENERATEMIPMAPPROC glGenerateMipmap = NULL;

typedef GLXContext (*glXCreateContextAttribsARBProc)(Display*, GLXFBConfig, GLXContext, Bool, const int*);
//定数定義
#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600
#define MAX_BONE 100

//ボーン構造体
struct BONE
{
	mat4 bindPose;//初期ポーズ
	mat4 newPose;//現在のポーズ
	unsigned int numChild;//子の数
	int childIndex[50];//自分の子の”インデックス" 50個まで
	char name[256];

	BONE()
	{
		memset(this,0, sizeof(BONE));
		bindPose=mat4(1.0);
		newPose=mat4(1.0);
	}
};
//頂点構造体
struct MY_VERTEX
{
	vec3 pos;//頂点位置
	vec3 normal;//頂点法線
	vec2 uv;//UV座標
	GLuint boneIndex[4];//ボーン　番号
	GLfloat boneWeight[4];//ボーン　重み
	MY_VERTEX()
	{
		memset(this,0, sizeof(MY_VERTEX));
	}
};
//オリジナル　マテリアル構造体
struct MY_MATERIAL
{
	vec4 Ka;//アンビエント
	vec4 Kd;//ディフューズ
	vec4 Ks;//スペキュラー
	char texFileName[256];//テクスチャーファイル名
	GLuint texture;
	unsigned int numPolygons;//そのマテリアルであるポリゴン数
	MY_MATERIAL()
	{
		memset(this, 0,sizeof(MY_MATERIAL));
	}
};
//グローバル変数
Display *g_dpy;
Window g_root;
XVisualInfo* g_vi;
Window g_win;
GLXContext g_ctx;
GLuint g_simpleShader=0;
GLuint g_vao = 0;
GLuint g_vertexBuffer=0;
GLuint g_indexBuffer[100];
int g_numBone;
BONE* g_BoneArray;
MY_MATERIAL g_material[100];
int g_numMaterial=0;

//関数プロトタイプ
bool InitOGL();
GLuint LoadShaders(const char * vertex_file_path, const char * fragment_file_path);
GLuint CreateTextureFromBMPFile(const char * imagepath);
void Render();
bool ReadSkinInfo(MY_VERTEX* pvVB);
void SetNewPoseMatrices(int iFrame);
mat4 GetCurrentPoseMatrix(int index);
mat4 GetBindPoseMatrix(int index);
char* GetBoneNames(int iBoneIndex);

bool InitX(char* szFileName);
int GetNumVertices();
int GetNumFaces();
int GetNumMaterials();
int GetNumUVs();
int GetNumBones();
int GetNumBoneVertices(int iBoneIndex);
int GetBoneVerticesIndices(int iBoneIndex, int iIndexInGroup);
double GetBoneVerticesWeights(int iBoneIndex, int iIndexInGroup);
vec3 GetVertex(int iIndex);
vec3 GetNormal(int iIndex);
vec2 GetUV(int iIndex);
int GetIndex(int iIndex);
vec4 GetAmbient(int iIndex);
vec4 GetDiffuse(int iIndex);
vec4 GetSpecular(int iIndex);
char* GetTexturePath(int index);
float GetSpecularPower(int iIndex);
int GeFaceMaterialIndex(int iFaceIndex);
int GetFaceVertexIndex(int iFaceIndex, int iIndexInFace);
mat4 GetBindPose(int iBoneIndex);
mat4 GetNewPose(int iBoneIndex, int iFrame);
int GetNumChildBone(int iBoneIndex);
int GetChildBoneIndex(int iBoneIndex, int iIndex);
char* GetBoneName(int iBoneIndex);
int GetNumBonesInVertex(int iVertexIndex);

//
//
//メイン関数 X11ウィンドウ作成とメッセージループ
int main(int argc, char *argv[])
{
    //X11ウィンドウの作成
    {
        //ディスプレイの取得
        g_dpy = XOpenDisplay(NULL);
        if(!g_dpy)
        {
            printf("Xサーバーに接続できませんでしたn");
            return 0;
        }
        //ビジュアルの取得
        GLint att[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
        g_vi = glXChooseVisual(g_dpy, 0, att);
        if(g_vi == NULL)
        {
            printf("\マッチするビジュアルが見つかりませんでした\n");
            return 0;
        }
        //カラーマップの作成
        Colormap cmap;
        g_root = DefaultRootWindow(g_dpy);
        cmap = XCreateColormap(g_dpy, g_root, g_vi->visual, AllocNone);

        //X11ウィンドウを作成
        XSetWindowAttributes swa;
        swa.colormap = cmap;
        swa.event_mask =  KeyPressMask;
        g_win = XCreateWindow(g_dpy, g_root, 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, g_vi->depth, InputOutput, g_vi->visual, CWColormap | CWEventMask, &swa);

        //作成したX11ウィンドウをマップする
        XMapWindow(g_dpy, g_win);
        //ウィンドウ名を指定
        XStoreName(g_dpy, g_win, "Skinned Mesh (define in a external file) ");
    }

    //OpenGLコンテキスト作成とVAO,VBO作成
    InitOGL();

	//メッセージループ
	bool loop=true;
    while(loop)
    {
        while(XPending(g_dpy))
        {
            XEvent xev;
            XNextEvent(g_dpy, &xev);
            if(xev.type == KeyPress) loop=false;
        }
        Render();
        glXSwapBuffers(g_dpy, g_win);
    }
    //終了処理
    glXMakeCurrent(g_dpy, None, NULL);
    glXDestroyContext(g_dpy, g_ctx);
    XDestroyWindow(g_dpy, g_win);
    XCloseDisplay(g_dpy);

    return 0;
}
//
//
//OpenGLコンテキスト作成とVAO,VBO作成等
bool InitOGL()
{
    //OpenGLバージョンを指定してコンテキストを作成
    int numFBC = 0;
    GLint visualAtt[] =
    {
        GLX_RENDER_TYPE, GLX_RGBA_BIT,
        GLX_DRAWABLE_TYPE, GLX_WINDOW_BIT,
        GLX_DOUBLEBUFFER, True,
        GLX_RED_SIZE, 1,
        GLX_GREEN_SIZE, 1,
        GLX_BLUE_SIZE, 1,
        GLX_DEPTH_SIZE, 1,
        GLX_STENCIL_SIZE, 1,
        None
    };

    GLXFBConfig *fbc = glXChooseFBConfig(g_dpy, DefaultScreen(g_dpy), visualAtt, &numFBC);

    if (!fbc)
    {
        printf("フレームバッファ設定を取得できませんでした\n");
        return false;
    }
    glXCreateContextAttribsARBProc glXCreateContextAttribsARB = (glXCreateContextAttribsARBProc) glXGetProcAddress((const GLubyte *) "glXCreateContextAttribsARB");
    if (!glXCreateContextAttribsARB)
    {
        fprintf(stderr, "glXCreateContextAttribsARB関数が見つかりません\n");
        XFree(fbc);
        return false;
    }
     static int contextAttribs[] =
    {
        GLX_CONTEXT_MAJOR_VERSION_ARB, 4,//希望のメジャーバージョン番号を指定
        GLX_CONTEXT_MINOR_VERSION_ARB, 0,//希望のマイナーバージョン番号を指定
        GLX_CONTEXT_PROFILE_MASK_ARB, GLX_CONTEXT_CORE_PROFILE_BIT_ARB,
        None
    };
    //コンテキスト作成
    g_ctx = glXCreateContextAttribsARB(g_dpy, *fbc, NULL, True, contextAttribs);
    XFree(fbc);
    if (!g_ctx)
    {
        printf("コンテキスト作成に失敗しました\n");
        return false;
    }
    //このコンテキストをカレントにする
    glXMakeCurrent(g_dpy, g_win, g_ctx);

    //Init (OpenGL) Eextensions関数の実際のアドレスをポインターに代入して完了　ステップ③
	glCreateProgram = (PFNGLCREATEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glCreateProgram");
	glGenVertexArrays = (PFNGLGENVERTEXARRAYSPROC)glXGetProcAddress((const GLubyte*)"glGenVertexArrays");
	glBindVertexArray = (PFNGLBINDVERTEXARRAYPROC)glXGetProcAddress((const GLubyte*)"glBindVertexArray");
	glCreateShader = (PFNGLCREATESHADERPROC)glXGetProcAddress((const GLubyte*)"glCreateShader");
	glShaderSource = (PFNGLSHADERSOURCEPROC)glXGetProcAddress((const GLubyte*)"glShaderSource");
	glCompileShader = (PFNGLCOMPILESHADERPROC)glXGetProcAddress((const GLubyte*)"glCompileShader");
	glGetShaderiv = (PFNGLGETSHADERIVPROC)glXGetProcAddress((const GLubyte*)"glGetShaderiv");
	glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetShaderInfoLog");
	glAttachShader = (PFNGLATTACHSHADERPROC)glXGetProcAddress((const GLubyte*)"glAttachShader");
	glLinkProgram = (PFNGLLINKPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glLinkProgram");
	glGetProgramiv = (PFNGLGETPROGRAMIVPROC)glXGetProcAddress((const GLubyte*)"glGetProgramiv");
	glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC)glXGetProcAddress((const GLubyte*)"glGetProgramInfoLog");
	glGenBuffers = (PFNGLGENBUFFERSPROC)glXGetProcAddress((const GLubyte*)"glGenBuffers");
	glBindBuffer = (PFNGLBINDBUFFERPROC)glXGetProcAddress((const GLubyte*)"glBindBuffer");
	glBufferData = (PFNGLBUFFERDATAPROC)glXGetProcAddress((const GLubyte*)"glBufferData");
	glUseProgram = (PFNGLUSEPROGRAMPROC)glXGetProcAddress((const GLubyte*)"glUseProgram");
	glEnableVertexAttribArray = (PFNGLENABLEVERTEXATTRIBARRAYPROC)glXGetProcAddress((const GLubyte*)"glEnableVertexAttribArray");
	glVertexAttribPointer = (PFNGLVERTEXATTRIBPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribPointer");
	glGetAttribLocation = (PFNGLGETATTRIBLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetAttribLocation");
    glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC)glXGetProcAddress((const GLubyte*)"glGetUniformLocation");
	glUniformMatrix4fv = (PFNGLUNIFORMMATRIX4FVPROC)glXGetProcAddress((const GLubyte*)"glUniformMatrix4fv");
	glUniform4f = (PFNGLUNIFORM4FPROC)glXGetProcAddress((const GLubyte*)"glUniform4f");
	glUniform1i = (PFNGLUNIFORM1IARBPROC)glXGetProcAddress((const GLubyte*)"glUniform1i");
	glVertexAttribIPointer = (PFNGLVERTEXATTRIBIPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribIPointer");
   	glGenerateMipmap = (PFNGLGENERATEMIPMAPPROC)glXGetProcAddress((const GLubyte*)"glGenerateMipmap");
	glUniform1i = (PFNGLUNIFORM1IARBPROC)glXGetProcAddress((const GLubyte*)"glUniform1i");
	glVertexAttribIPointer = (PFNGLVERTEXATTRIBIPOINTERPROC)glXGetProcAddress((const GLubyte*)"glVertexAttribIPointer");

    //シェーダー作成
    g_simpleShader = LoadShaders("SimpleVS.glsl", "SimpleFS.glsl");
    //VAO
    glGenVertexArrays(1, &g_vao);
	glBindVertexArray(g_vao);

	//ジオメトリ作成
	{
		//X_LoaderにXファイルからすべての情報を読み込ませる
		if (!InitX((char*)"hand_tex.x"))
		{
			return false;
		}
		//頂点数、インデックス数
		int numVertices = GetNumVertices();
		int numIndices = GetNumFaces()*3;
		int numFace = GetNumFaces();

		//頂点データ
		MY_VERTEX* pv = new MY_VERTEX[numVertices];
		//位置、法線、uvを詰め込む
		for (int i = 0; i < numVertices; i++)
		{
			//位置
			pv[i].pos = GetVertex(i);
			//法線
			pv[i].normal = GetNormal(i);
			//テクスチャー座標
			pv[i].uv = GetUV(i);
		}
		//ボーン情報を詰め込む
		ReadSkinInfo(pv);

		//頂点バッファ
		glGenBuffers(1, &g_vertexBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
		glBufferData(GL_ARRAY_BUFFER, numVertices*sizeof(MY_VERTEX), (void*)pv, GL_STATIC_DRAW);

		//マテリアル読み込み
		g_numMaterial = GetNumMaterials();

		//マテリアルの数だけインデックスバッファーを作成
		int numMaterials = GetNumMaterials();

		for (int i = 0; i < numMaterials; i++)
		{
			//環境光
			g_material[i].Ka = GetAmbient(i);
			//拡散反射光
			g_material[i].Kd = GetDiffuse(i);
			//鏡面反射光
			g_material[i].Ks = GetSpecular(i);
			//テクスチャー
			strcpy(g_material[i].texFileName, GetTexturePath(i));
			//テクスチャーを作成
			g_material[i].texture =CreateTextureFromBMPFile(g_material[i].texFileName);

			//そのマテリアルであるインデックス配列内の開始インデックスを調べる　さらにインデックスの個数も調べる
			int cnt = 0;
			int* pIndex = new int[numFace*3];//とりあえず、メッシュ内のポリゴン数でメモリ確保（個々のポリゴングループはかならずこれ以下になるが）

			for (int k = 0; k < numFace; k++)
			{
				if (i == GeFaceMaterialIndex(k))//もし i == k番目のポリゴンのマテリアル番号 なら
				{
					pIndex[cnt] = GetFaceVertexIndex(k, 0);//k番目のポリゴンを作る頂点のインデックス　1個目
					pIndex[cnt + 1] = GetFaceVertexIndex(k, 1);//2個目
					pIndex[cnt + 2] = GetFaceVertexIndex(k, 2);//3個目
					cnt += 3;
				}
			}
			if (cnt > 0)
			{
				glGenBuffers(1, &g_indexBuffer[i]);
				glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indexBuffer[i]);
				glBufferData(GL_ELEMENT_ARRAY_BUFFER, cnt*sizeof(int), pIndex, GL_STATIC_DRAW);
			}
			delete pIndex;

			g_material[i].numPolygons = cnt / 3;//そのマテリアル内のポリゴン数
		}
	}
	// テクスチャーのサンプラーとテクスチャーユニットを紐付ける
	GLuint texture = glGetUniformLocation(g_simpleShader, "g_textureSampler");
	glUniform1i(texture, 0);
}
//
//
//
GLuint CreateTextureFromBMPFile(const char * imagepath)
{
	FILE * fp=fopen(imagepath, "rb");
	if (!fp)
	{
		printf("ファイルを開けません\n");
		return 0;
	}
	//BMPヘッダーは54バイト
	unsigned char header[54];
	if (fread(header, 1, 54, fp) != 54)
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	//ビットマップであれば"BM"ヘッダーがあるはず
	if (header[0] != 'B' || header[1] != 'M')
	{
		printf("ビットマップファイルではありません\n");
		return 0;
	}
	// 24bitカラーかどうか
	if (*(int*)&(header[30]) != 0 || *(int*)&(header[28]) != 24)
	{
		printf("24ビットカラーではありません");
		return 0;
	}
	//ピクセルデータを読み込む
	unsigned int width = *(int*)&(header[18]);
	unsigned int height = *(int*)&(header[22]);
	unsigned int size = *(int*)&(header[34]);

	unsigned char* data = new unsigned char[size];
	fread(data, 1, size, fp);
	//この時点でもはやファイルは不要
	fclose(fp);

	//空（から）のテクスチャーを作成
	GLuint id;
	glGenTextures(1, &id);
	//テクスチャーをバインドして、これからそのテクスチャーにデータを送り込むことを指示する
	glBindTexture(GL_TEXTURE_2D, id);
	// ピクセルデータをテクスチャーに送り込む
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_BGR, GL_UNSIGNED_BYTE, data);
	//この時点でメモリー上のデータは不要になる
	delete data;
	//テクスチャーの仕様を設定
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glGenerateMipmap(GL_TEXTURE_2D);

	return id;
}
//
//
//
bool ReadSkinInfo(MY_VERTEX* pvVB)
{
	//Xから抽出すべき情報は、頂点ごとのボーンインデックス、頂点ごとのボーンウェイト、バインド行列、ポーズ行列　の4項目

	int i, k, m, n;
	int iNumVertex = GetNumVertices();//頂点数

	//ボーン	の個数を得る
	g_numBone = GetNumBones();
	//それぞれのボーンに影響を受ける頂点を調べる　そこから逆に、頂点ベースでボーンインデックス・重みを整頓する
	for (i = 0; i < g_numBone; i++)
	{
		int iNumIndex = GetNumBoneVertices(i);//このボーンに影響を受ける頂点数
		int* piIndex = new int[iNumIndex];
		for (k = 0; k < iNumIndex; k++) piIndex[k] = GetBoneVerticesIndices(i, k);
		GLfloat* pdWeight = new GLfloat[iNumIndex];
		for (k = 0; k < iNumIndex; k++) pdWeight[k] = GetBoneVerticesWeights(i, k);
		//頂点側からインデックスをたどって、頂点サイドで整理する
		for (k = 0; k < iNumIndex; k++)
		{
			//XやCGソフトがボーン4本以内とは限らない。5本以上の場合は、重みの大きい順に4本に絞る

			//ウェイトの大きさ順にソート（バブルソート）
			for (m = 4; m > 1; m--)
			{
				for (n = 1; n < m; n++)
				{
					if (pvVB[piIndex[k]].boneWeight[n - 1] < pvVB[piIndex[k]].boneWeight[n])
					{
						float tmp = pvVB[piIndex[k]].boneWeight[n - 1];
						pvVB[piIndex[k]].boneWeight[n - 1] = pvVB[piIndex[k]].boneWeight[n];
						pvVB[piIndex[k]].boneWeight[n] = tmp;
						int itmp = pvVB[piIndex[k]].boneIndex[n - 1];
						pvVB[piIndex[k]].boneIndex[n - 1] = pvVB[piIndex[k]].boneIndex[n];
						pvVB[piIndex[k]].boneIndex[n] = itmp;
					}
				}
			}
			//ソート後は、最後の要素に一番小さいウェイトが入っているはず。
			bool flag = false;
			for (m = 0; m < 4; m++)
			{
				if (pvVB[piIndex[k]].boneWeight[m] == 0)
				{
					flag = true;
					pvVB[piIndex[k]].boneIndex[m] = i;
					pvVB[piIndex[k]].boneWeight[m] = pdWeight[k];
					break;
				}
			}
			if (flag == false)
			{
				pvVB[piIndex[k]].boneIndex[3] = i;
				pvVB[piIndex[k]].boneWeight[3] = pdWeight[k];
				break;
			}

		}
		delete piIndex;
		delete pdWeight;
	}
	//
	//ボーンを生成
	g_BoneArray = new BONE[g_numBone];
	for (int i = 0; i < g_numBone; i++)
	{
		//子があれば、子へのインデックスを取得
		g_BoneArray[i].numChild = GetNumChildBone(i);
		for (int k = 0; k < g_BoneArray[i].numChild; k++)
		{
			g_BoneArray[i].childIndex[k] = GetChildBoneIndex(i, k);
		}
	}
	//ポーズ行列を得る 初期ポーズ
	for (i = 0; i < g_numBone; i++)
	{
		g_BoneArray[i].bindPose = GetBindPose(i);
	}

	return true;
}
//
//
//シェーダー読み込みコンパイル
GLuint LoadShaders(const char* vertexFilename, const char* fragmentFilename)
{
	GLuint vertexShader = glCreateShader(GL_VERTEX_SHADER);
	GLuint fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
	//バーテックスシェーダのコードを読み込む
	std::string VertexShaderCode;
	std::ifstream VertexShaderStream(vertexFilename, std::ios::in);
	if (VertexShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << VertexShaderStream.rdbuf();
		VertexShaderCode = sstr.str();
		VertexShaderStream.close();
	}
	else
	{
		printf("ファイルが見つかりません\n");
		return 0;
	}
	//フラグメントシェーダーのコードを読み込む
	std::string FragmentShaderCode;
	std::ifstream FragmentShaderStream(fragmentFilename, std::ios::in);
	if (FragmentShaderStream.is_open())
	{
		std::stringstream sstr;
		sstr << FragmentShaderStream.rdbuf();
		FragmentShaderCode = sstr.str();
		FragmentShaderStream.close();
	}
	GLint Result = GL_FALSE;
	int infoLength;
	//バーテックスシェーダをコンパイル
	char const * VertexSourcePointer = VertexShaderCode.c_str();
	glShaderSource(vertexShader, 1, &VertexSourcePointer, NULL);
	glCompileShader(vertexShader);

	glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(vertexShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(vertexShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//フラグメントシェーダーをコンパイル
	char const * FragmentSourcePointer = FragmentShaderCode.c_str();
	glShaderSource(fragmentShader, 1, &FragmentSourcePointer, NULL);
	glCompileShader(fragmentShader);

	glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &Result);
	glGetShaderiv(fragmentShader, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> errorMessage(infoLength + 1);
		glGetShaderInfoLog(fragmentShader, infoLength, NULL, &errorMessage[0]);
		printf(&errorMessage[0]);
	}
	//バーテックスシェーダーとフラグメントシェーダーを「プログラムオブジェクト」に関連付ける
	GLuint program = glCreateProgram();
	glAttachShader(program, vertexShader);
	glAttachShader(program, fragmentShader);
	glLinkProgram(program);

	glGetProgramiv(program, GL_LINK_STATUS, &Result);
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLength);
	if (infoLength > 0)
	{
		std::vector<char> ProgramErrorMessage(infoLength + 1);
		glGetProgramInfoLog(program, infoLength, NULL, &ProgramErrorMessage[0]);
		printf(&ProgramErrorMessage[0]);
	}

	return program;
}
//
//
//ボーンを次のポーズ位置にセットする
void SetNewPoseMatrices(int iFrame)
{
	for (int i = 0; i < g_numBone; i++)
	{
		g_BoneArray[i].newPose = GetNewPose(i, iFrame);
	}
}
//
//
//次の（現在の）ポーズ行列を返す
mat4 GetCurrentPoseMatrix(int index)
{
	mat4 ret = g_BoneArray[index].newPose*g_BoneArray[index].bindPose;

	return ret;
}
//
//
//バインドポーズ行列を返す
mat4 GetBindPoseMatrix(int index)
{
	return g_BoneArray[index].bindPose;
}
//
//
//
char* GetBoneNames(int iBoneIndex)
{
	return GetBoneName(iBoneIndex);
}
//
//
//描画
void Render()
{
    //Depthテスト有効にして陰面消去
	glEnable(GL_DEPTH_TEST);
    //ビューポートをセット
    XWindowAttributes gwa;
    XGetWindowAttributes(g_dpy, g_win, &gwa);
    glViewport(0, 0, gwa.width, gwa.height);
    //クリア
    glClearColor(0.0f, 0.0f, 0.4f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //これ以降使うシェーダーのセット
    glUseProgram(g_simpleShader);
    glBindVertexArray(g_vao);
    //VBOをセット
	glBindBuffer(GL_ARRAY_BUFFER, g_vertexBuffer);
	int offset = 0;
	//シェーダーに頂点内の「位置成分」の場所とサイズを教える
	GLint posAttrib = glGetAttribLocation(g_simpleShader, "g_position");
	glEnableVertexAttribArray(posAttrib);
	glVertexAttribPointer(posAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(MY_VERTEX), (void*)offset);
	//シェーダーに頂点内の「法線成分」の場所とサイズを教える
	GLint normalAttrib = glGetAttribLocation(g_simpleShader, "g_normal");
	glEnableVertexAttribArray(normalAttrib);
	offset += sizeof(vec3);
	glVertexAttribPointer(normalAttrib, 3, GL_FLOAT, GL_FALSE, sizeof(MY_VERTEX), (void*)offset);
	//シェーダーに頂点内の「テクスチャー座標成分」の場所とサイズを教える
	GLint uvAttrib = glGetAttribLocation(g_simpleShader, "g_uv");
	glEnableVertexAttribArray(uvAttrib);
	offset += sizeof(vec3);
	glVertexAttribPointer(uvAttrib, 2, GL_FLOAT, GL_FALSE, sizeof(MY_VERTEX), (void*)offset);
	//シェーダーに頂点内の「ボーン番号成分」の場所とサイズを教える
	GLint boneIndicesAttrib = glGetAttribLocation(g_simpleShader, "g_boneIndices");
	glEnableVertexAttribArray(boneIndicesAttrib);
	offset += sizeof(vec2);
	glVertexAttribIPointer(boneIndicesAttrib, 4, GL_UNSIGNED_INT, sizeof(MY_VERTEX), (void*)offset);
	//シェーダーに頂点内の「ボーンウェイト」の場所とサイズを教える
	GLint boneWeightAttrib = glGetAttribLocation(g_simpleShader, "g_boneWeights");
	glEnableVertexAttribArray(boneWeightAttrib);
	offset += sizeof(uvec4);
	glVertexAttribPointer(boneWeightAttrib, 4, GL_FLOAT, GL_FALSE, sizeof(MY_VERTEX), (void*)offset);

    //行列を渡す
    //ワールド行列
    mat4 world = mat4(1.0);
	static float angle = 0;
	angle+=0.01;
	world=rotate(world,angle,vec3(0,1,0));

	//ビュー行列
	mat4 view = lookAt(vec3(0, 0, 10), vec3(0, 0, 0), vec3(0, 1, 0));
	//プロジェクション行列
	mat4 proj = perspectiveFovRH(45.0, (double)WINDOW_WIDTH, (double)WINDOW_HEIGHT, 0.1, 10000.0);
	//３つの合成行列をシェーダーに渡す
	GLint matrix = glGetUniformLocation(g_simpleShader, "g_worldViewProjMat");
	mat4 wvp=proj*view*world;
	glUniformMatrix4fv(matrix, 1, GL_FALSE, (GLfloat*)&wvp);
	//法線を回転させるためにワールド行列だけを個別に渡す
	GLint worldMatrix = glGetUniformLocation(g_simpleShader, "g_worldMat");
	glUniformMatrix4fv(worldMatrix, 1, GL_FALSE, (GLfloat*)&(world));
	//シェーダーにライトの方向を渡す
	GLint light = glGetUniformLocation(g_simpleShader, "g_lightVector");
	glUniform4f(light, 0,0,1,0);

	//アニメーションフレームを進める　スキンを更新
	static int frame = 0;
	frame+=10;//アニメションが速すぎ理場合はこの値を少なく
	if (frame >= 3600) frame = 0;
	SetNewPoseMatrices(frame);
	mat4 boneMatrices[MAX_BONE];
	for (int i = 0; i < g_numBone; i++)
	{
		boneMatrices[i] = GetCurrentPoseMatrix(i);
	}
	//シェーダーにボーン行列群を渡す
	GLint boneMat = glGetUniformLocation(g_simpleShader, "g_boneMatrices");
	glUniformMatrix4fv(boneMat, g_numBone, GL_FALSE, (GLfloat*)boneMatrices);

	//描画

	//マテリアルの数だけ、それぞれのマテリアルのインデックスバッファ－を描画
	for (int i = 0; i < GetNumMaterials(); i++)
	{
		//使用されていないマテリアル対策
		if (g_material[i].numPolygons == 0)
		{
			continue;
		}
		//テクスチャーをセットする
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, g_material[i].texture);

		//VBOをセット
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_indexBuffer[i]);
		glDrawElements(GL_TRIANGLES, g_material[i].numPolygons *3, GL_UNSIGNED_INT, (void*)0);
	}
}
